@@ -10,7 +10,8 @@
 
 static MYSQL_ROW fix_fields_copy(STMT FAR *hstmt,MYSQL_ROW row);
 static int check_parameters(SQLHSTMT hstmt,SQLCHAR FAR *szTableQualifier,
-			    SQLSMALLINT cbTableQualifier,SQLCHAR FAR *szTableOwner,
+			    SQLSMALLINT cbTableQualifier,
+			    SQLCHAR FAR *szTableOwner,
 			    SQLSMALLINT cbTableOwner,SQLCHAR FAR *szTableName,
 			    SQLSMALLINT *cbTableName, char *table_name,
 			    bool no_wildcards);
@@ -38,7 +39,8 @@ const uint SQLTABLES_FIELDS=array_elements(SQLTABLES_values);
 */
 
 SQLRETURN SQL_API SQLTables(SQLHSTMT hstmt,
-			    SQLCHAR FAR *szTableQualifier,SQLSMALLINT cbTableQualifier,
+			    SQLCHAR FAR *szTableQualifier,
+			    SQLSMALLINT cbTableQualifier,
 			    SQLCHAR FAR *szTableOwner,SQLSMALLINT cbTableOwner,
 			    SQLCHAR FAR *szTableName,SQLSMALLINT cbTableName,
 			    SQLCHAR FAR *szTableType,SQLSMALLINT cbTableType)
@@ -54,9 +56,10 @@ SQLRETURN SQL_API SQLTables(SQLHSTMT hstmt,
 		      szTableType ? (char*) szTableType : "null"));
 
   my_SQLFreeStmt(hstmt,MYSQL_RESET);
-  if (!(type_buff=my_malloc(szTableType && cbTableType != SQL_NTS ? cbTableType+1 : 1,MYF(0))))
+  if (!(type_buff=my_malloc(szTableType && cbTableType != SQL_NTS ?
+			    cbTableType+1 : 1,MYF(0))))
   {
-    DBUG_RETURN(set_error(stmt->dbc,"S1001","Not enough memory",4001));
+    DBUG_RETURN(set_stmt_error(stmt,"S1001","Not enough memory",4001));
   }
 
   TableQualifier=fix_str((char FAR *) Qualifier_buff,szTableQualifier,
@@ -128,10 +131,13 @@ SQLRETURN SQL_API SQLTables(SQLHSTMT hstmt,
 
   /* This was a normal request for tables.  Return matching tables */
 
-  if (!(stmt->result=mysql_list_tables(&stmt->dbc->mysql,TableName)))
+  pthread_mutex_lock(&stmt->dbc->lock);
+  stmt->result=mysql_list_tables(&stmt->dbc->mysql,TableName);
+  pthread_mutex_unlock(&stmt->dbc->lock);
+  if (!stmt->result)
   {
     my_free(type_buff,MYF(0));
-    DBUG_RETURN(set_error(stmt->dbc,"S1000","Could not read table names",0));
+    DBUG_RETURN(set_stmt_error(stmt,"S1000","Could not read table names",0));
     DBUG_RETURN(SQL_ERROR);
   }
   stmt->order=       SQLTABLES_order;
@@ -187,10 +193,13 @@ const uint SQLCOLUMNS_FIELDS=array_elements(SQLCOLUMNS_values);
 
 
 SQLRETURN SQL_API SQLColumns(SQLHSTMT hstmt,
-			     SQLCHAR FAR *szTableQualifier, SQLSMALLINT cbTableQualifier,
-			     SQLCHAR FAR *szTableOwner, SQLSMALLINT cbTableOwner,
+			     SQLCHAR FAR *szTableQualifier,
+			     SQLSMALLINT cbTableQualifier,
+			     SQLCHAR FAR *szTableOwner,
+			     SQLSMALLINT cbTableOwner,
 			     SQLCHAR FAR *szTableName, SQLSMALLINT cbTableName,
-			     SQLCHAR FAR *szColumnName, SQLSMALLINT cbColumnName)
+			     SQLCHAR FAR *szColumnName,
+			     SQLSMALLINT cbColumnName)
 {
   char buff[80],table_name[NAME_LEN+1],column_name[NAME_LEN+1];
   ulong transfer_length,precision,display_size;
@@ -203,13 +212,18 @@ SQLRETURN SQL_API SQLColumns(SQLHSTMT hstmt,
   if (check_parameters(hstmt,szTableQualifier,cbTableQualifier,szTableOwner,
 		       cbTableOwner,szTableName,&cbTableName,table_name,1))
     return SQL_ERROR;
-  if (!(stmt->result=mysql_list_fields(&stmt->dbc->mysql,table_name,
-				       fix_str(column_name,szColumnName,
-					       cbColumnName))))
+  pthread_mutex_lock(&stmt->dbc->lock);
+  stmt->result=mysql_list_fields(&stmt->dbc->mysql,table_name,
+				 fix_str(column_name,szColumnName,
+					 cbColumnName));
+  if (!stmt->result)
   {
-    strmov(stmt->dbc->sqlstate,"S1000");
+    set_stmt_error(stmt,"S1000",mysql_error(&stmt->dbc->mysql),
+		   mysql_errno(&stmt->dbc->mysql));
+    pthread_mutex_unlock(&stmt->dbc->lock);
     DBUG_RETURN(SQL_ERROR);
   }
+  pthread_mutex_unlock(&stmt->dbc->lock);
   stmt->result_array= (char**) my_malloc(sizeof(char*)*SQLCOLUMNS_FIELDS*
 					 stmt->result->field_count,
 					 MYF(MY_FAE | MY_ZEROFILL));
@@ -281,11 +295,13 @@ MYSQL_FIELD SQLSTAT_fields[] = {
 const uint SQLSTAT_FIELDS=array_elements(SQLSTAT_fields);
 
 SQLRETURN SQL_API SQLStatistics(SQLHSTMT hstmt,
-			      SQLCHAR FAR *szTableQualifier,
-			      SQLSMALLINT cbTableQualifier,
-			      SQLCHAR FAR *szTableOwner,SQLSMALLINT cbTableOwner,
-			      SQLCHAR FAR *szTableName,SQLSMALLINT cbTableName,
-			      SQLUSMALLINT fUnique,SQLUSMALLINT fAccuracy)
+				SQLCHAR FAR *szTableQualifier,
+				SQLSMALLINT cbTableQualifier,
+				SQLCHAR FAR *szTableOwner,
+				SQLSMALLINT cbTableOwner,
+				SQLCHAR FAR *szTableName,
+				SQLSMALLINT cbTableName,
+				SQLUSMALLINT fUnique,SQLUSMALLINT fAccuracy)
 {
   char buff[100],table_name[NAME_LEN+1];
   STMT FAR *stmt=(STMT FAR*) hstmt;
@@ -294,15 +310,20 @@ SQLRETURN SQL_API SQLStatistics(SQLHSTMT hstmt,
   if (check_parameters(hstmt,szTableQualifier,cbTableQualifier,szTableOwner,
 		       cbTableOwner,szTableName,&cbTableName,table_name,1))
     DBUG_RETURN(SQL_ERROR);
-  remove_escape(table_name);
+  myodbc_remove_escape(&stmt->dbc->mysql,table_name);
   strxmov(buff,"show keys from ",table_name,NullS);
+
+  pthread_mutex_lock(&stmt->dbc->lock);
   if (check_if_server_is_alive(stmt->dbc) ||
       mysql_query(&stmt->dbc->mysql,buff) ||
       !(stmt->result=mysql_store_result(&stmt->dbc->mysql)))
   {
-    strmov(stmt->dbc->sqlstate,"S1000");
+    set_stmt_error(stmt,"S1000",mysql_error(&stmt->dbc->mysql),
+		   mysql_errno(&stmt->dbc->mysql));
+    pthread_mutex_unlock(&stmt->dbc->lock);
     DBUG_RETURN(SQL_ERROR);
   }
+  pthread_mutex_unlock(&stmt->dbc->lock);
   int2str(SQL_INDEX_OTHER,SS_type,10);
   stmt->order=       SQLSTAT_order;
   stmt->order_count= array_elements(SQLSTAT_order);
@@ -342,8 +363,8 @@ SQLRETURN SQL_API SQLTablePrivileges(SQLHSTMT hstmt,
 				     SQLSMALLINT cbTableName)
 {
   DBUG_ENTER("SQLTablePrivileges");
-  DBUG_RETURN(set_error(((STMT FAR*) hstmt)->dbc,"08002",
-			"MyODBC doesn't support this yet",4000));
+  DBUG_RETURN(set_stmt_error(((STMT FAR*) hstmt),"08002",
+			     "MyODBC doesn't support this yet",4000));
 }
 
 
@@ -360,8 +381,8 @@ SQLRETURN SQL_API SQLColumnPrivileges(SQLHSTMT hstmt,
 				      SQLSMALLINT cbColumnName)
 {
   DBUG_ENTER("SQLColumnPrivileges");
-  DBUG_RETURN(set_error(((STMT FAR*) hstmt)->dbc,"IM001",
-			"MyODBC doesn't support this yet",4000));
+  DBUG_RETURN(set_stmt_error(((STMT FAR*) hstmt),"IM001",
+			     "MyODBC doesn't support this yet",4000));
 }
 
 
@@ -410,11 +431,16 @@ SQLRETURN SQL_API SQLSpecialColumns(SQLHSTMT hstmt,SQLUSMALLINT fColType,
 		       cbTableOwner,szTableName,&cbTableName,table_name,1))
     DBUG_RETURN(SQL_ERROR);
 
-  if (!(stmt->result=mysql_list_fields(&stmt->dbc->mysql,table_name,0)))
+  pthread_mutex_lock(&stmt->dbc->lock);
+  stmt->result=mysql_list_fields(&stmt->dbc->mysql,table_name,0);
+  if (!stmt->result)
   {
-    strmov(stmt->dbc->sqlstate,"S1000");
+    set_stmt_error(stmt,"S1000",mysql_error(&stmt->dbc->mysql),
+		   mysql_errno(&stmt->dbc->mysql));
+    pthread_mutex_unlock(&stmt->dbc->lock);
     return(SQL_ERROR);
   }
+  pthread_mutex_unlock(&stmt->dbc->lock);
 
   if (fColType == SQL_ROWVER)
   {						/* Find possible timestamp */
@@ -458,10 +484,11 @@ SQLRETURN SQL_API SQLSpecialColumns(SQLHSTMT hstmt,SQLUSMALLINT fColType,
 
   if (fColType != SQL_BEST_ROWID)
   {
-    strmov(stmt->dbc->sqlstate,"S1000");
-    DBUG_RETURN(SQL_ERROR);
+    DBUG_RETURN(set_stmt_error(stmt,"S1000",
+			       "Unsupported argument to SQLSpecialColumns",
+			       4000));
   }
-    
+
   /*
    * The optimal set of columns for identifing a row is either
    * the primary key, or if there is no primary key, then
@@ -498,7 +525,7 @@ SQLRETURN SQL_API SQLSpecialColumns(SQLHSTMT hstmt,SQLUSMALLINT fColType,
        no primary or unique key */
     if (!primary_key)
       continue;
-#endif    
+#endif
     field_count++;
     sprintf(buff,"%d",SQL_SCOPE_SESSION);
     row[0]=strdup_root(alloc,buff);
@@ -541,10 +568,12 @@ MYSQL_FIELD SQLPRIM_KEYS_fields[] = {
 const uint SQLPRIM_KEYS_FIELDS=array_elements(SQLPRIM_KEYS_fields);
 
 SQLRETURN SQL_API SQLPrimaryKeys(SQLHSTMT hstmt,
-			       SQLCHAR FAR *szTableQualifier,
-			       SQLSMALLINT cbTableQualifier,
-			       SQLCHAR FAR *szTableOwner,SQLSMALLINT cbTableOwner,
-			       SQLCHAR FAR *szTableName,SQLSMALLINT cbTableName)
+				 SQLCHAR FAR *szTableQualifier,
+				 SQLSMALLINT cbTableQualifier,
+				 SQLCHAR FAR *szTableOwner,
+				 SQLSMALLINT cbTableOwner,
+				 SQLCHAR FAR *szTableName,
+				 SQLSMALLINT cbTableName)
 {
   char buff[100],table_name[NAME_LEN+1],**data;
   STMT FAR *stmt=(STMT FAR*) hstmt;
@@ -555,15 +584,19 @@ SQLRETURN SQL_API SQLPrimaryKeys(SQLHSTMT hstmt,
   if (check_parameters(hstmt,szTableQualifier,cbTableQualifier,szTableOwner,
 		       cbTableOwner,szTableName,&cbTableName,table_name,1))
     DBUG_RETURN(SQL_ERROR);
-  remove_escape(table_name);
+  myodbc_remove_escape(&stmt->dbc->mysql,table_name);
   strxmov(buff,"show keys from ",table_name,NullS);
+  pthread_mutex_lock(&stmt->dbc->lock);
   if (check_if_server_is_alive(stmt->dbc) ||
       mysql_query(&stmt->dbc->mysql,buff) ||
       !(stmt->result=mysql_store_result(&stmt->dbc->mysql)))
   {
-    strmov(stmt->dbc->sqlstate,"S1000");
+    set_stmt_error(stmt,"S1000",mysql_error(&stmt->dbc->mysql),
+		   mysql_errno(&stmt->dbc->mysql));
+    pthread_mutex_unlock(&stmt->dbc->lock);
     DBUG_RETURN(SQL_ERROR);
   }
+  pthread_mutex_unlock(&stmt->dbc->lock);
   stmt->result_array= (char**) my_malloc(sizeof(char*)*SQLPRIM_KEYS_FIELDS*
 					 (ulong) stmt->result->row_count,
 					 MYF(MY_FAE | MY_ZEROFILL));
@@ -645,7 +678,7 @@ SQLRETURN SQL_API SQLProcedures(SQLHSTMT hstmt,
 			      SQLCHAR FAR *szProcOwner,SQLSMALLINT cbProcOwner,
 			      SQLCHAR FAR *szProcName,SQLSMALLINT cbProcName)
 {
-  return set_error(((STMT FAR*)hstmt)->dbc,"08002",
+  return set_stmt_error(((STMT FAR*) hstmt),"08002",
 		   "MyODBC doesn't support this yet",4000);
 }
 
@@ -653,20 +686,25 @@ SQLRETURN SQL_API SQLProcedures(SQLHSTMT hstmt,
 //  Have DBMS set up result set of ProcedureColumns.
 
 SQLRETURN SQL_API SQLProcedureColumns(SQLHSTMT hstmt,
-				    SQLCHAR FAR *szProcQualifier,
-				    SQLSMALLINT cbProcQualifier,
-				    SQLCHAR FAR *szProcOwner,SQLSMALLINT cbProcOwner,
-				    SQLCHAR FAR *szProcName,SQLSMALLINT cbProcName,
-				    SQLCHAR FAR *szColumnName,SQLSMALLINT cbColumnName)
+				      SQLCHAR FAR *szProcQualifier,
+				      SQLSMALLINT cbProcQualifier,
+				      SQLCHAR FAR *szProcOwner,
+				      SQLSMALLINT cbProcOwner,
+				      SQLCHAR FAR *szProcName,
+				      SQLSMALLINT cbProcName,
+				      SQLCHAR FAR *szColumnName,
+				      SQLSMALLINT cbColumnName)
 {
-  return set_error(((STMT FAR*)hstmt)->dbc,"08002",
+  return set_stmt_error(((STMT FAR*) hstmt),"08002",
 		   "MyODBC doesn't support this yet",4000);
 }
 
 
 static int check_parameters(SQLHSTMT hstmt,SQLCHAR FAR *szTableQualifier,
-			    SQLSMALLINT cbTableQualifier,SQLCHAR FAR *szTableOwner,
-			    SQLSMALLINT cbTableOwner, SQLCHAR FAR *szTableName,
+			    SQLSMALLINT cbTableQualifier,
+			    SQLCHAR FAR *szTableOwner,
+			    SQLSMALLINT cbTableOwner,
+			    SQLCHAR FAR *szTableName,
 			    SQLSMALLINT *cbTableName,
 			    char *table_name,bool no_wildcards)
 {
@@ -681,8 +719,8 @@ static int check_parameters(SQLHSTMT hstmt,SQLCHAR FAR *szTableQualifier,
     {
       DBUG_PRINT("error",("szTableQualifier: '%s'  szTableOwner: '%s'  res: '%s'",
 			   szTableQualifier,szTableOwner,res));
-      return set_error(((STMT FAR*) hstmt)->dbc,
-  		       "S1C00","Can't use tablequalifer or owners",0);
+      return set_stmt_error(((STMT FAR*) hstmt),
+			    "S1C00","Can't use tablequalifer or owners",0);
     }
   }
 #endif
@@ -695,11 +733,11 @@ static int check_parameters(SQLHSTMT hstmt,SQLCHAR FAR *szTableQualifier,
   if (*cbTableName == SQL_NTS)
     *cbTableName=strlen(szTableName);
   if ((uint) *cbTableName > NAME_LEN)
-    return set_error(((STMT FAR*) hstmt)->dbc,"SC1090","Invalid table name",0);
+    return set_stmt_error(((STMT FAR*) hstmt),"SC1090","Invalid table name",0);
   strmake(table_name,szTableName,*cbTableName);
   if (no_wildcards && (!table_name[0] || strchr(table_name,'%')))
-    return(set_error(((STMT FAR*) hstmt)->dbc,"S1C00",
-		     "Can't use wildcards in table name",0));
+    return(set_stmt_error(((STMT FAR*) hstmt),"S1C00",
+			  "Can't use wildcards in table name",0));
   my_SQLFreeStmt(hstmt,MYSQL_RESET);
   return 0;
 }
