@@ -86,7 +86,7 @@ SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
 
   dbc=(DBC FAR*) *phdbc;
   dbc->env= (ENV*) henv;
-  dbc->mysql.net.fd = -1;			/* Marker if open */
+  dbc->mysql.net.vio = 0;			/* Marker if open */
   dbc->flag=0;
   dbc->stmt_options.max_rows= dbc->stmt_options.max_length= 0L;
   dbc->stmt_options.bind_type=SQL_BIND_BY_COLUMN;
@@ -94,6 +94,8 @@ SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC FAR *phdbc)
   dbc->stmt_options.cursor_type=SQL_CURSOR_FORWARD_ONLY;  /* ODBC default */
   dbc->login_timeout=0;
   dbc->last_query_time= (time_t) time((time_t*) 0);
+  dbc->txn_isolation=DEFAULT_TXN_ISOLATION;
+  pthread_mutex_init(&dbc->lock,NULL);
   return SQL_SUCCESS;
 }
 
@@ -108,6 +110,7 @@ SQLRETURN SQL_API SQLFreeConnect(SQLHDBC hdbc)
   my_free(dbc->server,MYF(MY_ALLOW_ZERO_PTR));
   my_free(dbc->user,MYF(MY_ALLOW_ZERO_PTR));
   my_free(dbc->password,MYF(MY_ALLOW_ZERO_PTR));
+  pthread_mutex_destroy(&dbc->lock);
 
 #ifndef _UNIX_
   GlobalUnlock(GlobalHandle((HGLOBAL) hdbc));
@@ -143,7 +146,7 @@ ulong get_client_flag(MYSQL *mysql, ulong option_flag,uint connect_timeout,
     client_flag |=  CLIENT_COMPRESS;
   if (option_flag & FLAG_IGNORE_SPACE)
     client_flag |=  CLIENT_IGNORE_SPACE;
-#ifdef __WIN32__
+#ifdef __WIN__
   if (option_flag & FLAG_NAMED_PIPE)
     mysql_options(mysql,MYSQL_OPT_NAMED_PIPE,NullS);
 #endif
@@ -159,6 +162,19 @@ ulong get_client_flag(MYSQL *mysql, ulong option_flag,uint connect_timeout,
 }
 
 
+/* Simple help functions to copy arguments if given */
+
+void copy_if_not_empty(char *to,uint max_length, char *from,int length)
+{
+  if (from)
+  {
+    if (length == SQL_NTS)
+      length=max_length-1;
+    strmake(to,from,length);
+  }
+}
+
+
 /******************************************************************************
 *	Connect to mysql server
 ******************************************************************************/
@@ -169,27 +185,32 @@ SQLRETURN SQL_API SQLConnect(SQLHDBC hdbc, SQLCHAR FAR *szDSN,
 			     SQLCHAR FAR *szAuthStr, SQLSMALLINT cbAuthStr)
 {
   char host[64],user[64],passwd[64],dsn[NAME_LEN+1],database[NAME_LEN+1],
-       port[10],flag[10],init_stmt[256],*passwd_ptr,*dsn_ptr;
+       port[10],flag[10],init_stmt[256],*dsn_ptr;
   char  szTRACE[FILENAME_MAX+1]  = "";
-  
+
   uint port_nr=0;
   ulong flag_nr,client_flag;
 
   DBC FAR *dbc=(DBC FAR*) hdbc;
   DBUG_ENTER("SQLConnect");
 
-  if (dbc->mysql.net.fd >= 0)
+  if (dbc->mysql.net.vio != 0)
   {
-    DBUG_RETURN(set_error(dbc,"08002","Connection in use",0));
+    DBUG_RETURN(set_dbc_error(dbc,"08002","Connection in use",0));
   }
   dbc->sqlstate[0]=0;
 
   dsn_ptr=fix_str(dsn,szDSN,cbDSN);
 
-  if(dsn_ptr && !dsn_ptr[0]) {
-    DBUG_RETURN(set_error(dbc,"S1090","Invalid DSN specified",0));
+  if (dsn_ptr && !dsn_ptr[0])
+  {
+    DBUG_RETURN(set_dbc_error(dbc,"S1090","Invalid DSN specified",0));
   }
 
+  SQLGetPrivateProfileString(dsn_ptr,"user",
+ 			     "", user, sizeof(user),"ODBC.INI");
+  SQLGetPrivateProfileString(dsn_ptr,"password",
+			     "", passwd, sizeof(passwd),"ODBC.INI");
   SQLGetPrivateProfileString(dsn_ptr,"server",
 			     "localhost", host, sizeof(host),"ODBC.INI");
   SQLGetPrivateProfileString(dsn_ptr,"database",
@@ -204,73 +225,46 @@ SQLRETURN SQL_API SQLConnect(SQLHDBC hdbc, SQLCHAR FAR *szDSN,
 			     "", init_stmt, sizeof(init_stmt),"ODBC.INI");
 
   SQLGetPrivateProfileString( dsn_ptr, "TRACE", "", szTRACE, sizeof(szTRACE),
-"ODBC.INI" );
- if (szTRACE[0] == '1' || toupper(szTRACE[0]) == 'Y' ||
-     toupper(szTRACE[0]) == 'O')
- {
-   char  szTRACEFILE[FILENAME_MAX+1] = "";
-   char  szMYODBC_LOG[FILENAME_MAX+20]= "";
-   SQLGetPrivateProfileString( dsn_ptr, "TRACEFILE", "", szTRACEFILE,
-			       sizeof(szTRACEFILE), "ODBC.INI" );
-   if (*szTRACEFILE)
-   {
-     sprintf(szMYODBC_LOG, "d:t:S:A,%s", szTRACEFILE);
-     DBUG_POP();
-     DBUG_PUSH(szMYODBC_LOG);
-   }
- }
-
-/* OBSOLETE
-  strmov(database,szDSN);
+			      "ODBC.INI" );
+  if (szTRACE[0] == '0' || szTRACE[0] == 'N' || szTRACE[0] == 'n' ||
+      ((szTRACE[0] == 'O' || szTRACE[0] == 'o') &&
+       (szTRACE[1] == 'N' || szTRACE[1] == 'n')))
   {
-    char *cp;
-    strmov(host, (cp = getenv("MYSQL_HOST")) ? cp : "localhost");
+    char  szTRACEFILE[FILENAME_MAX+1] = "";
+    char  szMYODBC_LOG[FILENAME_MAX+20]= "";
+    SQLGetPrivateProfileString( dsn_ptr, "TRACEFILE", "", szTRACEFILE,
+				sizeof(szTRACEFILE), "ODBC.INI" );
+    if (*szTRACEFILE)
+    {
+      sprintf(szMYODBC_LOG, "d:t:S:A,%s", szTRACEFILE);
+      DBUG_POP();
+      DBUG_PUSH(szMYODBC_LOG);
+    }
   }
-  flag_nr=0;
-  init_stmt[0]=0;
-*/
-
   client_flag=get_client_flag(&dbc->mysql,flag_nr,(uint) dbc->login_timeout,
 			      init_stmt);
-  passwd_ptr=fix_str(passwd,(char FAR*) szAuthStr,cbAuthStr);
-  if (!passwd_ptr[0])		/* Use my.cnf if empty password ! */
-    passwd_ptr=0;
-#if MYSQL_VERSION_ID >= 32200
-  if (!mysql_real_connect(&dbc->mysql,host,fix_str(user,(char FAR *) szUID,
-						   cbUID),
-			  passwd_ptr,
+ 
+  copy_if_not_empty(passwd,sizeof(passwd), (char FAR*) szAuthStr,cbAuthStr);
+  copy_if_not_empty(user, sizeof(user), (char FAR *) szUID, cbUID);
+
+  if (!mysql_real_connect(&dbc->mysql,host,user,passwd[0] ? passwd : 0,
 			  database, port_nr,
 			  NullS, (uint) client_flag))
-  {
-    strmov(dbc->sqlstate,"S1000");
-    translate_error(dbc);
-    DBUG_RETURN(SQL_ERROR);
-  }
-#else
-  if (!mysql_real_connect(&dbc->mysql,host,fix_str(user,(char FAR *) szUID,
-						   cbUID),
-			  passwd_ptr,
-			  port_nr, NullS, (uint) client_flag))
-  {
-    strmov(dbc->sqlstate,"S1000");
-    translate_error(dbc);
-    DBUG_RETURN(SQL_ERROR);
-  }
-  if (mysql_select_db(&dbc->mysql,database))
-  {
-    strmov(dbc->sqlstate,"08004");
-    mysql_close(&dbc->mysql);
-    DBUG_RETURN(SQL_ERROR);
-  }
-#endif
-  dbc->dsn=my_strdup(database,MYF(MY_WME));
-  dbc->database=my_strdup(database,MYF(MY_WME));
-  dbc->server=my_strdup(host,MYF(MY_WME));
-  dbc->user=my_strdup(user,MYF(MY_WME));
-  dbc->password=my_strdup(passwd,MYF(MY_WME));
-  dbc->port=port_nr;
-  dbc->flag=flag_nr;
-  DBUG_RETURN(SQL_SUCCESS);
+ {
+   set_dbc_error(dbc, "S1000", mysql_error(&dbc->mysql),
+		 mysql_errno(&dbc->mysql));
+   translate_error(dbc->sqlstate,"S1000",mysql_errno(&dbc->mysql));
+   DBUG_RETURN(SQL_ERROR);
+ }
+ dbc->dsn =dsn_ptr ? my_strdup(dsn_ptr, MYF(MY_WME)) : 
+                     my_strdup(database,MYF(MY_WME));
+ dbc->database=my_strdup(database,MYF(MY_WME));
+ dbc->server=my_strdup(host,MYF(MY_WME));
+ dbc->user=my_strdup(user,MYF(MY_WME));
+ dbc->password=my_strdup(passwd,MYF(MY_WME));
+ dbc->port=port_nr;
+ dbc->flag=flag_nr;
+ DBUG_RETURN(SQL_SUCCESS);
 }
 
 
@@ -281,35 +275,49 @@ SQLRETURN SQL_API SQLConnect(SQLHDBC hdbc, SQLCHAR FAR *szDSN,
 //	string to the user that is the string that was eventually used to
 //	establish the connection.
 
-#ifdef _UNIX_					/* By Tsurng-Chen Chern */
-char *my_strtok(char *s1, char *s2)
+#ifdef _UNIX_
+
+static char *my_strtok(char *s1, pchar separator, char **save)
 {
-   char *start;
-   static char *p;
+  reg1 char *rtnval,*end;
 
-   p = (s1 ? s1 : p + 1);
-   if ((!p) || (!*p))
-      return(NULL);
-
-   start = p;
-   while ((*p) && (!strchr(s2, *p)))
-      p++;
-   *p = 0;
-   return(start);
+  rtnval = NULL;
+  if (s1 != NULL)
+    end = s1;
+  else
+    end= *save;
+  if (end != NULL && *end != 0)
+  {
+    rtnval=end;
+    do
+    {
+      if (*end++ == separator)
+      {
+	if (separator)
+	  end[-1]=0;
+	else
+	  end--;			/* \0 as separator */
+	break;
+      }
+    } while (*end != 0);
+    *save=end;
+  }
+  return (rtnval);
 }
 #endif
 
 
-SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc,SQLHWND hwnd,SQLCHAR FAR *szConnStrIn,
-				   SQLSMALLINT cbConnStrIn,SQLCHAR FAR *szConnStrOut,
+SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc,SQLHWND hwnd,
+				   SQLCHAR FAR *szConnStrIn,
+				   SQLSMALLINT cbConnStrIn,
+				   SQLCHAR FAR *szConnStrOut,
 				   SQLSMALLINT cbConnStrOutMax,
 				   SQLSMALLINT FAR *pcbConnStrOut,
 				   SQLUSMALLINT fDriverCompletion)
 {
 #ifndef _UNIX_
-
   short   iRet;
-  BOOL    fPrompt = FALSE,maybe_prompt;
+  bool    fPrompt = FALSE,maybe_prompt;
   DBC FAR *dbc=(DBC FAR*) hdbc;
   GLOBALHANDLE hglbAttr;
   LPSETUPDLG lpsetupdlg;
@@ -339,8 +347,7 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc,SQLHWND hwnd,SQLCHAR FAR *szConn
     flag=(ulong) atol(lpsetupdlg->aAttr[KEY_FLAG].szAttr);
     if (fDriverCompletion == SQL_DRIVER_PROMPT && ! (flag & FLAG_NO_PROMPT) ||
 	(maybe_prompt &&
-	 (!lpsetupdlg->aAttr[KEY_DSN].szAttr[0] ||
-	  !lpsetupdlg->aAttr[KEY_SERVER].szAttr[0] ||
+	 (!lpsetupdlg->aAttr[KEY_SERVER].szAttr[0] ||
 	  !lpsetupdlg->aAttr[KEY_USER].szAttr[0])))
     {
       DBUG_PRINT("info",("flag: %d  dsn:  '%s'  server: '%s'  user: '%s'",
@@ -353,7 +360,7 @@ SQLRETURN SQL_API SQLDriverConnect(SQLHDBC hdbc,SQLHWND hwnd,SQLCHAR FAR *szConn
   }
 retry:
   if (fPrompt)
-  {
+  {		
     iRet = DialogBoxParam(s_hModule, MAKEINTRESOURCE(DRIVERCONNECT), hwnd,
 		          (DLGPROC) DriverConnectProc,
 		          (LONG)(LPSTR) lpsetupdlg);
@@ -371,7 +378,7 @@ retry:
     ulong tmp= ~0L;
     pos=strxmov(buff,
 		"DSN=",lpsetupdlg->aAttr[KEY_DSN].szAttr,
-		";DB=",lpsetupdlg->aAttr[KEY_DB].szAttr,
+		";DATABASE=",lpsetupdlg->aAttr[KEY_DB].szAttr,
 		";SERVER=",lpsetupdlg->aAttr[KEY_SERVER].szAttr,
 		";UID=",lpsetupdlg->aAttr[KEY_USER].szAttr,
 		";PWD=",lpsetupdlg->aAttr[KEY_PASSWORD].szAttr,
@@ -380,7 +387,7 @@ retry:
 		";STMT=",lpsetupdlg->aAttr[KEY_STMT].szAttr,
 		";",NullS);
     (void) copy_lresult((DBC FAR*) 0,szConnStrOut,cbConnStrOutMax,
-                        &length,buff,(uint) (pos-buff),0L,0L,&tmp,0);
+                        &length,buff, (pos-buff),0L,0L,&tmp,0);
     if (pcbConnStrOut)
       *pcbConnStrOut=(SQLSMALLINT) length;
     DBUG_PRINT("info",("Connect string out: %s",szConnStrOut));
@@ -406,9 +413,16 @@ retry:
     if (maybe_prompt && !fPrompt)
     {
       fPrompt=TRUE;
+      /* display the error message in the connection dialog in the next 
+         connection, so that, user know what is the mistake...
+      */
+      strxmov(lpsetupdlg->aAttr[KEY_ERROR].szAttr , 
+	      "[MySQL AB][MyODBC] ERROR : ",
+	      mysql_error(&dbc->mysql), NullS);
       goto retry;
     }
-    strmov(dbc->sqlstate,"S1000");
+    set_dbc_error(dbc,"S1000",mysql_error(&dbc->mysql),
+		  mysql_errno(&dbc->mysql));
     GlobalUnlock(hglbAttr);
     GlobalFree(hglbAttr);
     DBUG_RETURN(SQL_ERROR);
@@ -435,17 +449,17 @@ retry:
   char *KEY_DB=NULL;
   char *KEY_SOCKET=NULL;
   char *KEY_STMT=0;
-  
+
   char *tok[9];
   char *tok2=NULL;
   char *tok3=NULL;
   int i;
   uint client_flag=0;
-  
+
   char host[64],user[64],passwd[64],dsn[NAME_LEN+1],database[NAME_LEN+1],
        port[10],flag[10],socket[256],init_stmt[256],option[64];
 
-  BOOL    fPrompt = FALSE;
+  bool    fPrompt = FALSE;
   DBC FAR *dbc=(DBC FAR*) hdbc;
   DBUG_ENTER("SQLDriverConnect");
   DBUG_PRINT("enter",("fDriverCompletion: %d",fDriverCompletion));
@@ -455,23 +469,24 @@ retry:
     fPrompt = TRUE;
   else
   {
+    char *strtok_save;
     /* Check connection string for completeness. */
     /* Prompt if not all params			 */
 
     connstrin=(SQLCHAR FAR *) my_strdup(szConnStrIn,MYF(MY_WME));
 
-    tok[0]=my_strtok(connstrin,";");
+    tok[0]=my_strtok(connstrin,';', &strtok_save);
     for (i=1 ; i <= 7 ; i++)
-      tok[i]=my_strtok(NULL,";");
+      tok[i]=my_strtok(NULL,';',&strtok_save);
     tok[i]=NULL;
 
     for (i=0 ; tok[i] ; i++)
     {
-      if (tok2 = my_strtok(tok[i], "="))
+      if ((tok2 = my_strtok(tok[i], '=', &strtok_save)))
       {
 	if (strcasecmp("DSN", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0,&strtok_save)) && tok3[0])
 	    KEY_DSN = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -480,9 +495,10 @@ retry:
 	  }
 	}
 
-	if (strcasecmp("DB", tok2) == 0)
+	if (strcasecmp("DB", tok2) == 0 ||
+	    strcasecmp("DATABASE", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_DB = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -493,7 +509,7 @@ retry:
 
 	if (strcasecmp("UID", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_USER = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -504,7 +520,7 @@ retry:
 
 	if (strcasecmp("PWD", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_PASSWORD = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -515,7 +531,7 @@ retry:
 
 	if (strcasecmp("SERVER", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_SERVER = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -526,7 +542,7 @@ retry:
 
 	if (strcasecmp("PORT", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_PORT = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -537,7 +553,7 @@ retry:
 
 	if (strcasecmp("SOCKET", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_SOCKET = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -548,7 +564,7 @@ retry:
 
 	if (strcasecmp("OPTION", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_OPTION = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -559,7 +575,7 @@ retry:
 
 	if (strcasecmp("STMT", tok2) == 0)
 	{
-	  if ((tok3 = my_strtok(NULL, "\0")) && tok3[0])
+	  if ((tok3 = my_strtok(NULL, 0, &strtok_save)) && tok3[0])
 	    KEY_STMT = (char*) my_strdup(tok3, MYF(MY_WME));
 	  else
 	  {
@@ -638,7 +654,7 @@ retry:
   }
   if (fPrompt)
   {
-    DBUG_RETURN(set_error((DBC FAR*) hdbc,"S1000","No DSN entered",0));
+    DBUG_RETURN(set_dbc_error((DBC FAR*) hdbc,"S1000","No DSN entered",0));
   }
 
   if (szConnStrOut)
@@ -662,7 +678,6 @@ retry:
   client_flag=get_client_flag(&dbc->mysql,dbc->flag,(uint) dbc->login_timeout,
 			      KEY_STMT);
 
-#if MYSQL_VERSION_ID >= 32200
   if (!mysql_real_connect(&dbc->mysql,KEY_SERVER,
 			  KEY_USER,
 			  KEY_PASSWORD[0] ? KEY_PASSWORD : NullS,
@@ -671,27 +686,10 @@ retry:
 			  KEY_SOCKET[0] ? KEY_SOCKET : NullS,
 			  (uint) client_flag))
   {
-    strmov(dbc->sqlstate,"S1000");
+    set_dbc_error(dbc,"S1000",mysql_error(&dbc->mysql),
+		  mysql_errno(&dbc->mysql));
     DBUG_RETURN(SQL_ERROR);
   }
-#else
-  if (!mysql_real_connect(&dbc->mysql,KEY_SERVER,
-			  KEY_USER,
-			  KEY_PASSWORD[0] ? KEY_PASSWORD : NullS,
-                          dbc->port,
-			  KEY_SOCKET[0] ? KEY_SOCKET : NullS,
-			  (uint) client_flag))
-  {
-    strmov(dbc->sqlstate,"S1000");
-    DBUG_RETURN(SQL_ERROR);
-  }
-  if (mysql_select_db(&dbc->mysql,KEY_DB))
-  {
-    strmov(dbc->sqlstate,"08004");
-    mysql_close(&dbc->mysql);
-    DBUG_RETURN(SQL_ERROR);
-  }
-#endif
   dbc->dsn=my_strdup(KEY_DSN,MYF(MY_WME));
   dbc->database=my_strdup(KEY_DB,MYF(MY_WME));
   dbc->server=my_strdup(KEY_SERVER,MYF(MY_WME));
@@ -715,12 +713,14 @@ retry:
 
 
 SQLRETURN SQL_API SQLBrowseConnect(SQLHDBC hdbc,SQLCHAR FAR *szConnStrIn,
-				 SQLSMALLINT cbConnStrIn,
-				 SQLCHAR FAR *szConnStrOut,SQLSMALLINT cbConnStrOutMax,
-				 SQLSMALLINT FAR *pcbConnStrOut)
+				   SQLSMALLINT cbConnStrIn,
+				   SQLCHAR FAR *szConnStrOut,
+				   SQLSMALLINT cbConnStrOutMax,
+				   SQLSMALLINT FAR *pcbConnStrOut)
 {
   DBUG_ENTER("SQLBrowseConnect");
-  DBUG_RETURN(set_error((DBC FAR*) hdbc,"IM001","This doesn't work yet",4000));
+  DBUG_RETURN(set_dbc_error((DBC FAR*) hdbc,"IM001",
+			    "This doesn't work yet",4000));
 }
 
 
