@@ -9,6 +9,12 @@
 #include "myodbc.h"
 #include <m_ctype.h>
 
+uchar allowed_chars[]= {
+  '\307','\374','\351','\342','\344','\340','\345','\347','\352','\353',
+  '\350','\357','\356','\354','\304','\305','\311','\346','\306','\364',
+  '\366','\362','\373','\371','\377','\326','\334','\341','\355','\363',
+  '\372','\361','\321',0};
+
 SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 			     SQLPOINTER rgbInfoValue,
 			     SQLSMALLINT cbInfoValueMax,
@@ -17,6 +23,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   DBC FAR *dbc=(DBC FAR*) hdbc;
   char dummy2[16];
   SQLSMALLINT dummy;
+  bool using_323;
   DBUG_ENTER("SQLGetInfo");
   DBUG_PRINT("enter",("fInfoType: %d",fInfoType));
 
@@ -74,7 +81,8 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 			   - (char*) rgbInfoValue);
     break;
   case SQL_DBMS_VER:
-    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,MYSQL_SERVER_VERSION,
+    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,
+					 mysql_get_server_info(&dbc->mysql),
 					 cbInfoValueMax)
 				 - (char*) rgbInfoValue);
     break;
@@ -91,7 +99,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     if (dbc->flag & FLAG_NO_DEFAULT_CURSOR)
      *((long*) rgbInfoValue)&= ~ (long) SQL_FD_FETCH_PRIOR;
     *pcbInfoValue=sizeof(long);
-    break;
+    break; 
   case SQL_ODBC_VER:
     *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,DRIVER_VERSION,
 				   cbInfoValueMax)
@@ -103,9 +111,10 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     *pcbInfoValue=sizeof(SQLSMALLINT);
     break;
   case SQL_SERVER_NAME:
-    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,dbc->mysql.host_info,
-				   cbInfoValueMax)
-			   - (char*) rgbInfoValue);
+    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,
+					 dbc->mysql.host_info,
+					 cbInfoValueMax)
+				 - (char*) rgbInfoValue);
     break;
   case SQL_SEARCH_PATTERN_ESCAPE:
     *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,"\\",cbInfoValueMax)
@@ -123,11 +132,9 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_ROW_UPDATES:
   case SQL_ACCESSIBLE_PROCEDURES:
   case SQL_PROCEDURES:
-  case SQL_EXPRESSIONS_IN_ORDERBY:
   case SQL_NEED_LONG_DATA_LEN:
   case SQL_DATA_SOURCE_READ_ONLY:
   case SQL_ODBC_SQL_OPT_IEF:
-  case SQL_LIKE_ESCAPE_CLAUSE:
   case SQL_ORDER_BY_COLUMNS_IN_SELECT:
    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,"N",cbInfoValueMax)
 			   - (char*) rgbInfoValue);
@@ -135,20 +142,27 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_OUTER_JOINS:
      if (dbc->mysql.protocol_version == MYSQL_3_21_PROTOCOL)   /* MySQL 3.21 */
        *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,
-                                      (dbc->flag & FLAG_OLD_ODBC_PROG) ? "Y" : "F",
-                                       cbInfoValueMax)
-  			       - (char*) rgbInfoValue);
+					    (dbc->flag & FLAG_OLD_ODBC_PROG) ?
+					    "Y" : "F",
+					    cbInfoValueMax)
+				    - (char*) rgbInfoValue);
      else
-      *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,"N",cbInfoValueMax)
-  			      - (char*) rgbInfoValue);
+      *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,"N",
+					   cbInfoValueMax)
+				   - (char*) rgbInfoValue);
      break;
 #ifdef SQL_OJ_CAPABILITIES
   case SQL_OJ_CAPABILITIES:
-    *((SQLUINTEGER*) rgbInfoValue)= SQL_OJ_LEFT | SQL_OJ_NESTED | SQL_OJ_NOT_ORDERED |
-                                    SQL_OJ_INNER | SQL_OJ_ALL_COMPARISON_OPS;
+    using_323=is_prefix(mysql_get_server_info(&dbc->mysql),"3.23");
+    *((SQLUINTEGER*) rgbInfoValue)= (SQL_OJ_LEFT | SQL_OJ_NESTED |
+				     SQL_OJ_NOT_ORDERED |
+				     SQL_OJ_INNER | SQL_OJ_ALL_COMPARISON_OPS |
+				     (using_323 ? SQL_OJ_RIGHT : 0));
     *pcbInfoValue=sizeof(SQLUINTEGER);
     break;
 #endif
+  case SQL_LIKE_ESCAPE_CLAUSE:
+  case SQL_EXPRESSIONS_IN_ORDERBY:
   case SQL_COLUMN_ALIAS:
   case SQL_ACCESSIBLE_TABLES:
   case SQL_MULT_RESULT_SETS:
@@ -177,7 +191,7 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     break
 #endif
   case SQL_DEFAULT_TXN_ISOLATION:
-    *((long*) rgbInfoValue)= SQL_TXN_READ_UNCOMMITTED;
+    *((long*) rgbInfoValue)= DEFAULT_TXN_ISOLATION;
     *pcbInfoValue=sizeof(long);
     break;
 #ifdef SQL_DESCRIBE_PARAMETER
@@ -187,16 +201,26 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     break;
 #endif
   case SQL_TXN_ISOLATION_OPTION:
-    *((long*) rgbInfoValue)= SQL_TXN_READ_UNCOMMITTED;
+    if (!(dbc->mysql.server_capabilities & CLIENT_TRANSACTIONS) ||
+	(dbc->flag & FLAG_NO_TRANSACTIONS))
+      *((long*) rgbInfoValue)= SQL_TXN_READ_COMMITTED;
+    else
+      *((long*) rgbInfoValue)= (SQL_TXN_READ_COMMITTED |
+				SQL_TXN_READ_UNCOMMITTED |
+				SQL_TXN_REPEATABLE_READ |
+				SQL_TXN_SERIALIZABLE);
     *pcbInfoValue=sizeof(long);
     break;
   case SQL_IDENTIFIER_CASE:
     *((SQLSMALLINT*) rgbInfoValue)=SQL_IC_MIXED;
     *pcbInfoValue=sizeof(SQLSMALLINT);
     break;
+
   case SQL_IDENTIFIER_QUOTE_CHAR:
-    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue," ",cbInfoValueMax)
-			   - (char*) rgbInfoValue);
+    using_323=is_prefix(mysql_get_server_info(&dbc->mysql),"3.23");
+    *pcbInfoValue=(SQLSMALLINT)
+      (strmake((char*) rgbInfoValue, (using_323 ? "`" : " "), cbInfoValueMax)
+       - (char*) rgbInfoValue);
     break;
   case SQL_MAX_COLUMN_NAME_LEN:
    *((SQLSMALLINT*) rgbInfoValue)=NAME_LEN;
@@ -245,7 +269,10 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 			   - (char*) rgbInfoValue);
     break;
   case SQL_TXN_CAPABLE:
-    *((SQLSMALLINT*) rgbInfoValue)=SQL_TC_NONE;
+    if(dbc && !(dbc->flag & FLAG_NO_TRANSACTIONS))
+      *((SQLSMALLINT*) rgbInfoValue)=SQL_TC_DDL_COMMIT;
+    else
+      *((SQLSMALLINT*) rgbInfoValue)=SQL_TC_NONE;
     *pcbInfoValue=sizeof(SQLSMALLINT);
     break;
   case SQL_USER_NAME:
@@ -316,7 +343,6 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
   case SQL_CONVERT_BINARY:
   case SQL_CONVERT_VARBINARY:
   case SQL_CONVERT_LONGVARBINARY:
-  case SQL_POSITIONED_STATEMENTS:
   case SQL_LOCK_TYPES:
   case SQL_BOOKMARK_PERSISTENCE:
   case SQL_OWNER_USAGE:
@@ -328,10 +354,19 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     *pcbInfoValue=sizeof(long);
     break;
   case SQL_QUOTED_IDENTIFIER_CASE:
-    *((SQLUSMALLINT*) rgbInfoValue)=SQL_IC_MIXED;
+    *((SQLUSMALLINT*) rgbInfoValue)=SQL_IC_SENSITIVE;
     break;
   case SQL_POS_OPERATIONS:
-    *((long*) rgbInfoValue)=SQL_POS_POSITION;
+    *((long*) rgbInfoValue)= SQL_POS_POSITION |
+                             SQL_POS_UPDATE  |
+                             SQL_POS_DELETE |
+                             SQL_POS_REFRESH |
+                             SQL_POS_ADD;
+    *pcbInfoValue=sizeof(long);
+    break;
+  case SQL_POSITIONED_STATEMENTS:
+    *((long*) rgbInfoValue)= SQL_PS_POSITIONED_DELETE |
+                             SQL_PS_POSITIONED_UPDATE;
     *pcbInfoValue=sizeof(long);
     break;
   case SQL_ALTER_TABLE:
@@ -390,7 +425,8 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     *pcbInfoValue=sizeof(SQLSMALLINT);
     break;
   case SQL_GETDATA_EXTENSIONS:
-   *((long*) rgbInfoValue)= SQL_GD_ANY_COLUMN | SQL_GD_ANY_ORDER | SQL_GD_BOUND;
+   *((long*) rgbInfoValue)= SQL_GD_ANY_COLUMN | SQL_GD_ANY_ORDER |
+     SQL_GD_BOUND;
     *pcbInfoValue=sizeof(long);
     break;
   case SQL_STATIC_SENSITIVITY:
@@ -408,23 +444,25 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
     break;
   case SQL_KEYWORDS:
     *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,
-				   "UNIQUE,ZEROFILL,UNSIGNED,BIGINT,BLOB,TINYBLOB,MEDIMUMBLOB,LONGBLOB,MEDIUMINT,PROCEDURE,SHOW,LIMIT,DEFAULT,TABLES,REGEXP,RLIKE,KEYS,TINTTEXT,MEDIUMTEXT",
+				   "UNIQUE,ZEROFILL,UNSIGNED,BIGINT,BLOB,TINYBLOB,MEDIMUMBLOB,LONGBLOB,MEDIUMINT,PROCEDURE,SHOW,LIMIT,DEFAULT,TABLES,REGEXP,RLIKE,KEYS,TINYTEXT,MEDIUMTEXT",
 				   cbInfoValueMax)
 			   - (char*) rgbInfoValue);
     break;
   case SQL_SPECIAL_CHARACTERS:
   {
-    *pcbInfoValue=(SQLSMALLINT) (strmake((char*) rgbInfoValue,
-				   "",
-				   cbInfoValueMax)
-			   - (char*) rgbInfoValue);
+    /* "" */
+     *pcbInfoValue=(SQLSMALLINT)
+       (strmake((char*) rgbInfoValue,
+		(char*) allowed_chars,
+		cbInfoValueMax)
+	- (char*) rgbInfoValue);
      break;
   }
   default:
   {
     char buff[80];
     sprintf(buff,"Unsupported option: %d to SQLGetInfo",fInfoType);
-    DBUG_RETURN(set_error(dbc,"S1C00",buff,4000));
+    DBUG_RETURN(set_dbc_error(dbc,"S1C00",buff,4000));
   }
   }
   DBUG_RETURN(SQL_SUCCESS);
@@ -438,114 +476,140 @@ SQLRETURN SQL_API SQLGetInfo(SQLHDBC hdbc, SQLUSMALLINT fInfoType,
 */
 
 MYSQL_FIELD SQL_GET_TYPE_INFO_fields[] = {
-  {"Type_name","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,32,32,NOT_NULL_FLAG},
-  {"Data_Type","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
-  {"Precision","GetTypeInfo",NULL,FIELD_TYPE_LONG,10,10},
-  {"Literal_prefix","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,1,1},
-  {"Literal_suffix","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,1,1},
-  {"Create_params","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,15,15},
-  {"Nullable","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
-  {"Case_sensitive","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
-  {"Searchable","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
-  {"Unsigned_attribute","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
-  {"Money","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
-  {"Auto_increment","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
-  {"Local_type_name","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,60,60},
-  {"Minimum_scale","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
-  {"Maximum_scale","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
+  {"TYPE_NAME","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,32,32,NOT_NULL_FLAG},
+  {"DATA_TYPE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
+  {"PRECISION","GetTypeInfo",NULL,FIELD_TYPE_LONG,10,10},
+  {"LITERAL_PREFIX","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,2,2},
+  {"LITERAL_SUFFIX","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,2,2},
+  {"CREATE_PARAMS","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,15,15},
+  {"NULLABLE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
+  {"CASE_SENSITIVE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
+  {"SEARCHABLE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
+  {"UNSIGNED_ATTRIBUTE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
+  {"MONEY","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2,NOT_NULL_FLAG},
+  {"AUTO_INCREMENT","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
+  {"LOCAL_TYPE_NAME","GetTypeInfo",NULL,FIELD_TYPE_VAR_STRING,60,60},
+  {"MINIMUM_SCALE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
+  {"MAXIMUM_SCALE","GetTypeInfo",NULL,FIELD_TYPE_SHORT,2,2},
 };
 
 const uint SQL_GET_TYPE_INFO_FIELDS=array_elements(SQL_GET_TYPE_INFO_fields);
-#define MYSQL_DATA_TYPES 33
+#define MYSQL_DATA_TYPES 36
 
 char sql_searchable[6],sql_unsearchable[6],sql_nullable[6],sql_no_nulls[6],
      sql_bit[6],
      sql_tinyint[6],sql_smallint[6],sql_integer[6],sql_bigint[6],sql_float[6],
      sql_real[6],sql_double[6], sql_char[6],sql_varchar[6],sql_longvarchar[6],
      sql_timestamp[6], sql_decimal[6],sql_numeric[6],sql_varbinary[6],
-     sql_time[6],sql_date[6],sql_longvarbinary[6];
+     sql_time[6],sql_date[6],sql_binary[6],sql_longvarbinary[6];
 
 char *SQL_GET_TYPE_INFO_values[MYSQL_DATA_TYPES][15]=
 {
-  {"tinyint",sql_bit,"3",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Tiny integer",NULL,NULL},
+  /* SQL_BIT= -7 */
+  {"bit",sql_bit,"1",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","char(1)",NULL,NULL},
 
+  /* SQL_TINY= -6 */
   {"tinyint",sql_tinyint,"3",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Tiny integer",NULL,NULL},
+   "0",sql_searchable,"0","0","0","tinyint",NULL,NULL},
   {"tinyint unsigned",sql_tinyint,"3",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","Tiny integer unsigned",NULL,NULL},
+   "0",sql_searchable,"1","0","0","tinyint unsigned",NULL,NULL},
 
-  {"smallint",sql_smallint,"5",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Short integer",NULL,NULL},
-  {"smallint unsigned",sql_smallint,"5",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","Short integer unsigned",NULL,NULL},
-
-  {"middleint",sql_integer,"8",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Medium integer",NULL,NULL},
-  {"middleint unsigned",sql_integer,"8",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","Medium integer unsigned",NULL,NULL},
-
-  {"int",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","integer",NULL,NULL},
-  {"int unsigned",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","integer unsigned",NULL,NULL},
-
-  {"integer",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Integer",NULL,NULL},
-  {"integer unsigned",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","Integer unsigned",NULL,NULL},
-
-  {"bigint",sql_bigint,"20",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","Longlong integer",NULL,NULL},
+  /* SQL_BIGINT= -5 */
+  {"bigint",sql_bigint,"19",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","bigint",NULL,NULL},
   {"bigint unsigned",sql_bigint,"20",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"1","0","0","Longlong integer unsigned",NULL,NULL},
-
-  {"double",sql_double,"15",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","double","0","4"},
-  {"double",sql_float,"15",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","double","0","4"},
-
-  {"float",sql_real,"7",NULL,NULL,NULL,sql_nullable,
-   "0",sql_unsearchable,"0","0","0","float","0","2"},
-
-  {"decimal",sql_numeric,"15",NULL,NULL,"precision,scale",sql_nullable,
-   "0",sql_searchable,"0","0","0","double","0","6"},
-  {"decimal",sql_decimal,"15",NULL,NULL,"precision,scale",sql_nullable,
-   "0",sql_searchable,"0","0","0","double","0","6"},
-
-  {"char",sql_char,"255","'","'","maxlength",sql_nullable,
-   "0",sql_searchable,"0","0","0","string",NULL,NULL},
-  {"varchar",sql_varchar,"255","'","'","maxlength",sql_nullable,
-   "0",sql_searchable,"0","0","0","variable length string",NULL,NULL},
+   "0",sql_searchable,"1","0","0","bigint unsigned",NULL,NULL},
 
+  /* SQL_LONGVARBINARY = -3 */
+  {"long varbinary",sql_longvarbinary,"16777215","0x",NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","mediumblob",NULL,NULL},
   {"blob",sql_longvarbinary,"65535","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","binary large object (0-65535)",NULL,NULL},
-  {"text",sql_longvarchar,"65535","'","'",NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","large text object (0-65535)",NULL,NULL},
+  {"longblob",sql_longvarbinary,"2147483647","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","binary large object, use mediumblob instead",NULL,NULL},
   {"tinyblob",sql_longvarbinary,"255","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","binary large object (0-255) ",NULL,NULL},
   {"mediumblob",sql_longvarbinary,"16777215","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","binary large object",NULL,NULL},
-  {"mediumtext",sql_longvarchar,"16777215","'","'",NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","large text object",NULL,NULL},
-  {"longblob",sql_longvarbinary,"2147483647","'","'",NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","binary large object, use mediumblob instead",NULL,NULL},
 
-  {"timestamp",sql_timestamp,"14",NULL,NULL,NULL,sql_no_nulls,
-   "0",sql_searchable,"0","0","0","timestamp","0","0"},
-  {"time",sql_time,"6",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","time",NULL,NULL},
-  {"date",sql_date,"10",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","date",NULL,NULL},
-  {"datetime",sql_timestamp,"21",NULL,NULL,NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","datetime","0","0"},
+  /* SQL_LONGVARCHAR = -1 */
+  {"long varchar",sql_longvarchar,"16777215","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","mediumtext",NULL,NULL},
+  {"text",sql_longvarchar,"65535","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","text (0-65535)",NULL,NULL},
+  {"mediumtext",sql_longvarchar,"16777215","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","mediumtext",NULL,NULL},
+
+  /* SQL_CHAR = 1 */
+  {"char",sql_char,"255","'","'","length",sql_nullable,
+   "0",sql_searchable,"0","0","0","char",NULL,NULL},
+
+  /* SQL_DECIMAL = 3 */
+  {"numeric",sql_numeric,"19",NULL,NULL,"precision,scale",sql_nullable,
+   "0",sql_searchable,"0","0","0","numeric","0","19"},
+  {"decimal",sql_decimal,"19",NULL,NULL,"precision,scale",sql_nullable,
+   "0",sql_searchable,"0","0","0","decimal","0","19"},
+
+  /* SQL_INTEGER = 4 */
+  {"integer",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","integer",NULL,NULL},
+  {"integer unsigned",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"1","0","0","integer unsigned",NULL,NULL},
+  {"int",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","integer",NULL,NULL},
+  {"int unsigned",sql_integer,"10",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"1","0","0","integer unsigned",NULL,NULL},
+  {"mediumint",sql_integer,"7",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","Medium integer",NULL,NULL},
+  {"mediumint unsigned",sql_integer,"8",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"1","0","0","Medium integer unsigned",NULL,NULL},
+
+  /* SQL_SMALLINT = 5 */
+  {"smallint",sql_smallint,"5",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","smallint",NULL,NULL},
+  {"smallint unsigned",sql_smallint,"5",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"1","0","0","smallint unsigned",NULL,NULL},
   {"year",sql_smallint,"4",NULL,NULL,NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","year",NULL,NULL},
 
-  {"enum",sql_char,"255","'","'",NULL,sql_nullable,
+  /* SQL_FLOAT = 6 */
+  {"double",sql_float,"15",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","double","0","4"},
+
+  /* SQL_REAL = 7 */
+  {"float",sql_real,"7",NULL,NULL,NULL,sql_nullable,
+   "0",sql_unsearchable,"0","0","0","float","0","2"},
+
+  /* SQL_DOUBLE = 8 */
+  {"double",sql_double,"15",NULL,NULL,NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","double","0","4"},
+
+  /* SQL_DATE = 9 */
+  {"date",sql_date,"10","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","date",NULL,NULL},
+
+  /* SQL_TIME = 10 */
+  {"time",sql_time,"6","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","time",NULL,NULL},
+
+  /* SQL_TIMESTAMP = 11 */
+  {"datetime",sql_timestamp,"21","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","datetime","0","0"},
+  {"timestamp",sql_timestamp,"14","'","'",NULL,sql_no_nulls,
+   "0",sql_searchable,"0","0","0","timestamp","0","0"},
+
+  /* SQL_VARCHAR = 12 */
+
+  /* The following must be here to get Access to work */
+  {"text",sql_varchar,"255","'","'","",sql_nullable,
+   "0",sql_searchable,"0","0","0","text",NULL,NULL},
+  {"varchar",sql_varchar,"255","'","'","maxlength",sql_nullable,
+   "0",sql_searchable,"0","0","0","varchar",NULL,NULL},
+  {"enum",sql_varchar,"255","'","'",NULL,sql_nullable,
    "0",sql_searchable,"0","0","0","enum(value1,value2,value3...)",NULL,NULL},
-  {"set",sql_char,"255","'","'",NULL,sql_nullable,
-   "0",sql_searchable,"0","0","0","set(value1,value2,value3...)",NULL,NULL},
+  {"set",sql_varchar,"255","'","'",NULL,sql_nullable,
+   "0",sql_searchable,"0","0","0","set(value1,value2,value3...)",NULL,NULL}  
 };
 
 
@@ -610,11 +674,11 @@ void init_getfunctions(void)
   exists[SQL_API_SQLDESCRIBECOL]=TRUE;
   exists[SQL_API_SQLROWCOUNT]=TRUE;
   exists[SQL_API_SQLDISCONNECT]=TRUE;
-  exists[SQL_API_SQLSETCURSORNAME]=FALSE;
+  exists[SQL_API_SQLSETCURSORNAME]=TRUE;
   exists[SQL_API_SQLERROR]=TRUE;
   exists[SQL_API_SQLSETPARAM]=TRUE;
   exists[SQL_API_SQLEXECDIRECT]=TRUE;
-  exists[SQL_API_SQLTRANSACT]=TRUE;		/* Lie */
+  exists[SQL_API_SQLTRANSACT]=TRUE;
   exists[SQL_API_SQLEXECUTE]=TRUE;
 
   exists[SQL_API_SQLBINDPARAMETER]=TRUE;
@@ -672,6 +736,7 @@ void init_getfunctions(void)
   int2str(SQL_LONGVARCHAR,sql_longvarchar,-10);
   int2str(SQL_LONGVARBINARY,sql_longvarbinary,-10);
   int2str(SQL_VARBINARY,sql_varbinary,-10);
+  int2str(SQL_BINARY,sql_binary,-10);
   int2str(SQL_TIMESTAMP,sql_timestamp,-10);
   int2str(SQL_DATE,sql_date,-10);
   int2str(SQL_TIME,sql_time,-10);
