@@ -29,7 +29,7 @@ static SQLRETURN check_result(STMT FAR *stmt)
 
   switch (stmt->state) {
   case ST_UNKNOWN:
-    error=set_error(stmt->dbc,"24000","Invalid cursor state",0);
+    error=set_stmt_error(stmt,"24000","Invalid cursor state",0);
     break;
   case ST_PREPARED:
     if ((error=my_SQLExecute(stmt)) == SQL_SUCCESS)
@@ -42,6 +42,41 @@ static SQLRETURN check_result(STMT FAR *stmt)
    DBUG_RETURN(error);
 }
 
+SQLRETURN do_dummy_parambind(SQLHSTMT hstmt)
+{  
+  STMT FAR *stmt = (STMT FAR *)hstmt;
+
+  /* check for any un bounded parameter markers...
+     if any do the dummy binding
+  */
+  PARAM_BIND *param;
+  uint   nIndex;
+
+  for ( nIndex = 0; nIndex < stmt->param_count; nIndex++ )
+  {
+    param = dynamic_element(&stmt->params,nIndex,PARAM_BIND*);
+    if ( param->real_param_done != TRUE && param->used != 1)
+    {
+      SQLRETURN sqlRet;
+      SQLINTEGER  ndata;
+      SQLUSMALLINT ncol = (SQLUSMALLINT)nIndex+1;
+
+      /*
+        do the dummy bind temporarily to get the result set
+        and once everything is done, remove it
+      */
+      stmt->dummy_state = ST_DUMMY_PREPARED;
+      ndata = 0;
+      sqlRet = my_SQLBindParameter(hstmt, ncol, SQL_PARAM_INPUT,
+                  SQL_C_LONG,SQL_VARCHAR, 0, 0,
+                  &ndata,0,NULL);
+      if(sqlRet != SQL_SUCCESS && sqlRet != SQL_SUCCESS_WITH_INFO)
+         return(sqlRet);
+     }
+     stmt->dummy_state = ST_DUMMY_EXECUTED;
+   }
+  return(SQL_SUCCESS);
+}
 
 //      This returns the number of columns associated with the database
 //      attached to "hstmt".
@@ -52,8 +87,20 @@ SQLRETURN SQL_API SQLNumResultCols(SQLHSTMT  hstmt, SQLSMALLINT FAR *pccol)
   STMT FAR *stmt=(STMT FAR*) hstmt;
   DBUG_ENTER("SQLNumResultCols");
 
+  /*
+  here, when the stmt is prepared, do a dummy execute and get the resultset
+  and then set it back to PREPARED. Once prepare stmt supported in the server
+  then revert back
+  */
+  if (stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
+     (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED))
+  {
+    if(do_dummy_parambind(hstmt) != SQL_SUCCESS)
+       DBUG_RETURN(SQL_ERROR);
+  }
   if ((error=check_result(stmt)) != SQL_SUCCESS)
     DBUG_RETURN(error);
+
   if (!stmt->result)
     *pccol=0;					/* Not a select */
   else
@@ -66,10 +113,14 @@ SQLRETURN SQL_API SQLNumResultCols(SQLHSTMT  hstmt, SQLSMALLINT FAR *pccol)
 //      Return information about the database column the user wants
 //      information about.
 
-SQLRETURN SQL_API SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLCHAR FAR *szColName,
-			       SQLSMALLINT cbColNameMax, SQLSMALLINT FAR *pcbColName,
-			       SQLSMALLINT FAR *pfSqlType, SQLUINTEGER FAR *pcbColDef,
-			       SQLSMALLINT FAR *pibScale, SQLSMALLINT FAR *pfNullable)
+SQLRETURN SQL_API SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT icol,
+				 SQLCHAR FAR *szColName,
+				 SQLSMALLINT cbColNameMax,
+				 SQLSMALLINT FAR *pcbColName,
+				 SQLSMALLINT FAR *pfSqlType,
+				 SQLUINTEGER FAR *pcbColDef,
+				 SQLSMALLINT FAR *pibScale,
+				 SQLSMALLINT FAR *pfNullable)
 {
   SQLRETURN error;
   MYSQL_FIELD *field;
@@ -81,11 +132,11 @@ SQLRETURN SQL_API SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLCHAR FAR
   if ((error=check_result(stmt)) != SQL_SUCCESS)
     DBUG_RETURN(error);
   if (! stmt->result)
-    DBUG_RETURN(set_error(stmt->dbc,"07005","No result set",0));
+    DBUG_RETURN(set_stmt_error(stmt,"07005","No result set",0));
 
   mysql_field_seek(stmt->result,icol-1);
   if (!(field=mysql_fetch_field(stmt->result)))
-    DBUG_RETURN(set_error(stmt->dbc,"S1002","Invalid column number",0));
+    DBUG_RETURN(set_stmt_error(stmt,"S1002","Invalid column number",0));
   type=unireg_to_sql_datatype(stmt,field,0,&transfer_length,&precision,&display_size);
   if (pfSqlType)
     *pfSqlType=type;
@@ -105,24 +156,27 @@ SQLRETURN SQL_API SQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLCHAR FAR
     SQLRETURN error;
     if (!tmp)
     {
-      DBUG_RETURN(set_error(stmt->dbc,"S1001","Not enough memory",4001));
+      DBUG_RETURN(set_stmt_error(stmt,"S1001","Not enough memory",4001));
     }
     strxmov(tmp,field->table,".",field->name,NullS);
-    error=copy_result(stmt->dbc,szColName,cbColNameMax, pcbColName, tmp);
+    error=copy_result((DBC FAR *) 0, stmt, szColName, cbColNameMax,
+		      pcbColName, tmp);
     my_free((gptr) tmp,MYF(0));
     DBUG_RETURN(error);
   }
-  DBUG_RETURN(copy_result(stmt->dbc,szColName,cbColNameMax,pcbColName,
-			  field->name));
+  DBUG_RETURN(copy_result((DBC FAR *) 0, stmt, szColName, cbColNameMax,
+			  pcbColName, field->name));
 }
 
 
 
 //      Returns result column descriptor information for a result set.
 
-SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLUSMALLINT fDescType,
-				 SQLPOINTER rgbDesc, SQLSMALLINT cbDescMax,
-				 SQLSMALLINT FAR *pcbDesc, SQLINTEGER FAR *pfDesc)
+SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,
+				   SQLUSMALLINT fDescType,
+				   SQLPOINTER rgbDesc, SQLSMALLINT cbDescMax,
+				   SQLSMALLINT FAR *pcbDesc,
+				   SQLINTEGER FAR *pfDesc)
 {
   MYSQL_FIELD *field;
   STMT FAR *stmt=(STMT FAR*) hstmt;
@@ -138,7 +192,7 @@ SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLUSMALLINT
   if ((error=check_result(stmt)) != SQL_SUCCESS)
     DBUG_RETURN(error);
   if (! stmt->result)
-    DBUG_RETURN(set_error(stmt->dbc,"07005","No result set",0));
+    DBUG_RETURN(set_stmt_error(stmt,"07005","No result set",0));
 
   /* first handle some stupid cases that doesn't have anything to do
      with columns */
@@ -166,13 +220,13 @@ SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLUSMALLINT
 
   mysql_field_seek(stmt->result,icol-1);
   if (!(field=mysql_fetch_field(stmt->result)))
-    DBUG_RETURN(set_error(stmt->dbc,"S1002","Invalid column number",0));
+    DBUG_RETURN(set_stmt_error(stmt,"S1002","Invalid column number",0));
 
   switch (fDescType) {
   case SQL_COLUMN_LABEL:
   case SQL_COLUMN_NAME:
-    DBUG_RETURN(copy_result(stmt->dbc,(uchar*) rgbDesc,cbDescMax,pcbDesc,
-			    field->name));
+    DBUG_RETURN(copy_result((DBC FAR *) 0, stmt,(uchar*) rgbDesc,
+			    cbDescMax,pcbDesc, field->name));
   case SQL_COLUMN_TYPE:
     *pfDesc=unireg_to_sql_datatype(stmt,field,0,&transfer_length,&precision,
 				   &display_size);
@@ -201,8 +255,8 @@ SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLUSMALLINT
     *pcbDesc=sizeof(int);
     break;
   case SQL_COLUMN_NULLABLE:
-    *pfDesc= (field->flags & (NOT_NULL_FLAG | AUTO_INCREMENT_FLAG)) == NOT_NULL_FLAG ?
-             SQL_NO_NULLS : SQL_NULLABLE;
+    *pfDesc= ((field->flags & (NOT_NULL_FLAG | AUTO_INCREMENT_FLAG)) ==
+	      NOT_NULL_FLAG ? SQL_NO_NULLS : SQL_NULLABLE);
     *pcbDesc=sizeof(int);
     break;
   case SQL_COLUMN_SEARCHABLE:
@@ -233,31 +287,38 @@ SQLRETURN SQL_API SQLColAttributes(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLUSMALLINT
   case SQL_COLUMN_TYPE_NAME:
   {
     char buff[40];
-    (void) unireg_to_sql_datatype(stmt,field,buff,&transfer_length,&precision,&display_size);
-    DBUG_RETURN(copy_result(stmt->dbc,(uchar*) rgbDesc,cbDescMax,pcbDesc,
-			    buff));
+    (void) unireg_to_sql_datatype(stmt,field,buff,&transfer_length,
+				  &precision,&display_size);
+    DBUG_RETURN(copy_result((DBC FAR *) 0, stmt, (uchar*) rgbDesc, cbDescMax,
+			    pcbDesc, buff));
   }
   case SQL_COLUMN_OWNER_NAME:
   case SQL_COLUMN_QUALIFIER_NAME:
-    DBUG_RETURN(copy_result(stmt->dbc,(uchar*) rgbDesc,cbDescMax,pcbDesc,""));
+    DBUG_RETURN(copy_result((DBC FAR *) 0, stmt, (uchar*) rgbDesc, cbDescMax,
+			    pcbDesc,""));
+  case SQL_COLUMN_TABLE_NAME:
+    DBUG_RETURN(copy_result((DBC FAR *) 0, stmt, (uchar*) rgbDesc, cbDescMax,
+			    pcbDesc, field->table ? field->table : ""));
+  default:
+    DBUG_PRINT("warning",("Type: %d is not supported by MyODBC"));
   }
   DBUG_RETURN(SQL_SUCCESS);
 }
 
+/* Associate a user-supplied buffer with a database column. */
 
-//      Associate a user-supplied buffer with a database column.
-
-SQLRETURN SQL_API SQLBindCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLSMALLINT fCType, SQLPOINTER rgbValue,
-			   SQLINTEGER  cbValueMax, SQLINTEGER FAR *pcbValue)
+SQLRETURN SQL_API SQLBindCol(SQLHSTMT hstmt, SQLUSMALLINT icol,
+                             SQLSMALLINT fCType, SQLPOINTER rgbValue,
+                             SQLINTEGER  cbValueMax, SQLINTEGER FAR *pcbValue)
 {
   BIND *bind;
   STMT FAR *stmt=(STMT FAR*) hstmt;
   SQLRETURN error;
   DBUG_ENTER("SQLBindCol");
   DBUG_PRINT("enter",
-	     ("icol: %d  Type: %d  ValueMax: %ld  Valueptr: %lx  Value: %ld",
-	      icol,fCType,(long) cbValueMax,pcbValue,
-	      (long) (pcbValue ? *pcbValue : 0L)));
+             ("icol: %d  Type: %d  ValueMax: %ld  Valueptr: %lx  Value: %ld",
+              icol,fCType,(long) cbValueMax, pcbValue,
+              (long) (pcbValue ? *pcbValue : 0L)));
 
   icol--;
   /*
@@ -271,34 +332,44 @@ SQLRETURN SQL_API SQLBindCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLSMALLINT fCTy
     if (icol >= stmt->bound_columns)
     {
       if (!(stmt->bind=(BIND*) my_realloc((char*) stmt->bind,
-					  (icol+1)*sizeof(BIND),
-					  MYF(MY_ALLOW_ZERO_PTR |
-					      MY_FREE_ON_ERROR))))
+                                          (icol+1)*sizeof(BIND),
+                                          MYF(MY_ALLOW_ZERO_PTR |
+                                              MY_FREE_ON_ERROR))))
       {
-	stmt->bound_columns=0;
-	DBUG_RETURN(set_error(stmt->dbc,"S1001","Not enough memory",4001));
+        stmt->bound_columns=0;
+        DBUG_RETURN(set_stmt_error(stmt,"S1001","Not enough memory",4001));
       }
       bzero((gptr) (stmt->bind+stmt->bound_columns),
-		    (icol+1-stmt->bound_columns)*sizeof(BIND));
+      (icol+1-stmt->bound_columns)*sizeof(BIND));
       stmt->bound_columns=icol+1;
     }
   }
   else
   {
     /* Bind parameter to current set  ( The normal case ) */
-    if ((error=check_result(stmt)) != SQL_SUCCESS)
+    /* select stmt with parameters */
+    if (stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
+       (stmt->state != ST_PRE_EXECUTED || stmt->state != ST_EXECUTED))
+    {
+       if(do_dummy_parambind(hstmt) != SQL_SUCCESS)
+         DBUG_RETURN(SQL_ERROR);
+    }
+
+    if ((error = check_result(stmt)) != SQL_SUCCESS)
+    {
       DBUG_RETURN(error);
+    }
 
     if (!stmt->result || (uint) icol >= stmt->result->field_count)
     {
-      DBUG_RETURN(set_error(stmt->dbc,"S1002","Invalid column number",0));
+      DBUG_RETURN(set_stmt_error(stmt,"S1002","Invalid column number",0));
     }
     if (!stmt->bind)
     {
       if (!(stmt->bind=(BIND*) my_malloc(sizeof(BIND)*
-					 stmt->result->field_count,
-					 MYF(MY_ZEROFILL))))
-	DBUG_RETURN(set_error(stmt->dbc,"S1001","Not enough memory",4001));
+           stmt->result->field_count,
+           MYF(MY_ZEROFILL))))
+           DBUG_RETURN(set_stmt_error(stmt,"S1001","Not enough memory",4001));
       stmt->bound_columns=stmt->result->field_count;
     }
     mysql_field_seek(stmt->result,icol);
@@ -315,8 +386,10 @@ SQLRETURN SQL_API SQLBindCol(SQLHSTMT hstmt, SQLUSMALLINT icol, SQLSMALLINT fCTy
 }
 
 
-//      Returns data for bound columns in the current row ("stmt->iCursor"),
-//      advances the cursor.
+/*
+  Returns data for bound columns in the current row ("stmt->iCursor"),
+  advances the cursor.
+*/
 
 SQLRETURN SQL_API SQLFetch(SQLHSTMT hstmt)
 {
@@ -326,7 +399,7 @@ SQLRETURN SQL_API SQLFetch(SQLHSTMT hstmt)
   DBUG_ENTER("SQLFetch");
 
   if (!stmt->result)
-    DBUG_RETURN(set_error(stmt->dbc,"24000","Fetch without a SELECT",0));
+    DBUG_RETURN(set_stmt_error(stmt,"24000","Fetch without a SELECT",0));
   if (stmt->result_array)
   {
     if (stmt->current_row >= stmt->result->row_count)
@@ -363,7 +436,7 @@ SQLRETURN SQL_API SQLFetch(SQLHSTMT hstmt)
       {
 	stmt->getdata_offset= (ulong) ~0L;
  	if ((tmp_res=sql_get_data(stmt,bind->fCType,bind->field,bind->rgbValue,
-				  bind->cbValueMax,bind->pcbValue,
+				  bind->cbValueMax, bind->pcbValue,
 				  *values, lengths ? *lengths : *values ?
 				  strlen(*values) : 0) )
 	    != SQL_SUCCESS)
@@ -391,11 +464,11 @@ SQLRETURN SQL_API SQLFetch(SQLHSTMT hstmt)
 }
 
 
+	/* Returns result data for a single column in the current row. */
 
-//  Returns result data for a single column in the current row.
-
-SQLRETURN SQL_API SQLGetData(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLSMALLINT fCType,SQLPOINTER rgbValue,
-			   SQLINTEGER cbValueMax, SQLINTEGER FAR *pcbValue)
+SQLRETURN SQL_API SQLGetData(SQLHSTMT hstmt,SQLUSMALLINT icol,
+			     SQLSMALLINT fCType,SQLPOINTER rgbValue,
+			     SQLINTEGER cbValueMax, SQLINTEGER FAR *pcbValue)
 {
   STMT FAR *stmt=(STMT FAR*) hstmt;
   SQLRETURN result;
@@ -403,7 +476,7 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLSMALLINT fCType
 
   if (!stmt->result || !stmt->current_values)
   {
-    set_error(stmt->dbc,"24000","SQLGetData without a preceding SELECT",0);
+    set_stmt_error(stmt,"24000","SQLGetData without a preceding SELECT",0);
     DBUG_RETURN(SQL_ERROR);
   }
   icol--;		  /* Easier code if start from 0 */
@@ -441,10 +514,11 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT hstmt,SQLUSMALLINT icol,SQLSMALLINT fCType
 ** Get data.  rgbValue or pcbValue may be NULL pointers!
 */
 
-SQLRETURN SQL_API sql_get_data(STMT *stmt,SQLSMALLINT fCType,MYSQL_FIELD *field,
-			     SQLPOINTER rgbValue, SQLINTEGER  cbValueMax,
-			     SQLINTEGER FAR *pcbValue,
-			     char *value, uint length)
+SQLRETURN SQL_API sql_get_data(STMT *stmt,SQLSMALLINT fCType,
+			       MYSQL_FIELD *field,
+			       SQLPOINTER rgbValue, SQLINTEGER  cbValueMax,
+			       SQLINTEGER FAR *pcbValue,
+			       char *value, uint length)
 {
   DBC *dbc=stmt->dbc;
   long tmp;
@@ -558,7 +632,7 @@ SQLRETURN SQL_API sql_get_data(STMT *stmt,SQLSMALLINT fCType,MYSQL_FIELD *field,
       /*--------------------------------------------*/
       /* Check if it could be a date...... :)       */
       /*--------------------------------------------*/
-        if (value[4] == '-' && value[7] == '-' &&
+        if (length >= 10 && value[4] == '-' && value[7] == '-' &&
 	    (!value[10] || value[10] == ' '))
 	{
 	  *((long*) rgbValue)= ((long) atol(value)*10000L+
@@ -691,8 +765,10 @@ SQLRETURN SQL_API sql_get_data(STMT *stmt,SQLSMALLINT fCType,MYSQL_FIELD *field,
 }
 
 
-//  This determines whether there are more results sets available for
-//  the "hstmt".
+/*
+  This determines whether there are more results sets available for
+  the "hstmt".
+*/
 
 SQLRETURN SQL_API SQLMoreResults(SQLHSTMT  hstmt)
 {
@@ -701,8 +777,10 @@ SQLRETURN SQL_API SQLMoreResults(SQLHSTMT  hstmt)
 }
 
 
-//      This returns the number of rows affected byt an UPDATE, INSERT or
-//	DELETE statement
+/*
+  This returns the number of rows affected by an UPDATE, INSERT or
+  DELETE statement
+*/
 
 SQLRETURN SQL_API SQLRowCount(SQLHSTMT hstmt, SQLINTEGER FAR *pcrow)
 {
@@ -710,65 +788,23 @@ SQLRETURN SQL_API SQLRowCount(SQLHSTMT hstmt, SQLINTEGER FAR *pcrow)
   DBUG_ENTER("SQLRowCount");
   if (stmt->result)
   {
-	*pcrow=(SQLINTEGER) mysql_num_rows(stmt->result);
-	DBUG_PRINT("exit",("Rows in set: %ld",*pcrow));
+    *pcrow=(SQLINTEGER) mysql_affected_rows(&stmt->dbc->mysql);
+    DBUG_PRINT("exit",("Rows in set: %ld",*pcrow));
   }
   else
   {
-	*pcrow=(SQLINTEGER) mysql_affected_rows(&stmt->dbc->mysql);
-	DBUG_PRINT("exit",("Affected rows: %ld",*pcrow));
+    *pcrow=(SQLINTEGER) stmt->affected_rows;
+    DBUG_PRINT("exit",("Affected rows: %ld",*pcrow));
   }
   DBUG_RETURN(SQL_SUCCESS);
 }
 
+	/* This fetchs a block of data (rowset). */
 
-//      This positions the cursor within a block of data.
-
-SQLRETURN SQL_API SQLSetPos(SQLHSTMT hstmt, SQLUSMALLINT irow, SQLUSMALLINT fRefresh, SQLUSMALLINT fLock)
-{
-  STMT FAR *stmt=(STMT FAR*) hstmt;
-  DBUG_ENTER("SQLSetPos");
-  if (!stmt->result || !stmt->rows_found_in_set)
-  {
-    set_error(stmt->dbc,"S1010",
-	      "SQLSetPos without a preceding SQLExtendedFetch",0);
-    DBUG_RETURN(SQL_ERROR);
-  }
-  irow--;
-  if ((uint) irow >= stmt->rows_found_in_set)
-  {
-    DBUG_PRINT("error",("Tried to get irow: %d from a set of %d rows",
-                         irow,stmt->rows_found_in_set));
-    set_error(stmt->dbc,"S1107","SQLSetpos: Row is out of range",4000);
-    DBUG_RETURN(SQL_ERROR);
-  }
-  stmt->position_in_set=irow;
-  if (stmt->result_array)
-    stmt->current_values=stmt->result_array+((stmt->current_row+irow)*
-					      stmt->result->field_count);
-  else
-  {
-    mysql_data_seek(stmt->result,stmt->current_row+irow);
-    stmt->current_values=mysql_fetch_row(stmt->result);
-    if (stmt->fix_fields)
-      stmt->current_values=(*stmt->fix_fields)(stmt,stmt->current_values);
-    else
-      stmt->result_lengths=mysql_fetch_lengths(stmt->result);
-  }
-  if (fLock || fRefresh)
-  {
-    set_error(stmt->dbc,"S1109",
-	      "SQLSetPos with lock or refresh is not supported",4000);
-    DBUG_RETURN(SQL_ERROR);
-  }
-  DBUG_RETURN(SQL_SUCCESS);
-}
-
-
-//      This fetchs a block of data (rowset).
-
-SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLINTEGER irow,
-				 SQLUINTEGER FAR *pcrow, SQLUSMALLINT FAR *rgfRowStatus)
+SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType,
+				   SQLINTEGER irow,
+				   SQLUINTEGER FAR *pcrow,
+				   SQLUSMALLINT FAR *rgfRowStatus)
 {
   ulong cur_row,max_row,rows_to_fetch;
   uint i;
@@ -776,14 +812,24 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
   STMT FAR *stmt=(STMT FAR*) hstmt;
   MYSQL_ROW values;
   MYSQL_ROW_OFFSET save_position;
+  SQLUINTEGER dummy_pcrow;
   DBUG_ENTER("SQLExtendedFetch");
 
   if (!stmt->result)
-    DBUG_RETURN(set_error(stmt->dbc,"24000","Fetch without a SELECT",0));
+    DBUG_RETURN(set_stmt_error(stmt,"24000","Fetch without a SELECT",0));
   DBUG_PRINT("enter",
 	     ("fetchtype: %d  row: %ld  current top-row: %ld  rows_found: %ld",
 	      fFetchType,irow,stmt->current_row,stmt->rows_found_in_set));
 
+  if (stmt->stmt_options.cursor_type == SQL_CURSOR_FORWARD_ONLY &&
+      fFetchType != SQL_FETCH_NEXT &&
+      !(stmt->dbc->flag & FLAG_SAFE))
+    DBUG_RETURN(set_stmt_error(stmt,"S1106", /* HY106 in ODBC 3.0 */
+			       "Wrong fetchtype with FORWARD ONLY cursor",0));
+
+  if (!pcrow)
+    pcrow= &dummy_pcrow;
+
   max_row=(ulong) mysql_num_rows(stmt->result);
   stmt->last_getdata_col=(uint)  ~0;
   stmt->current_values=0;		/* For SQLGetData */
@@ -805,8 +851,8 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
   case SQL_FETCH_ABSOLUTE:
   if (irow == 0)
   {
-    *pcrow=stmt->rows_found_in_set=0;
-    stmt->current_row=0;		/* Fix for next fetch */
+    *pcrow=stmt->rows_found_in_set=0;    
+    stmt->current_row=0;    /* Fix for next fetch */
     mysql_data_seek(stmt->result,0L);
     DBUG_RETURN(SQL_NO_DATA_FOUND);
   }
@@ -823,7 +869,7 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
     cur_row=(ulong) stmt->current_row +(ulong) irow;
     break;
   default:
-    DBUG_RETURN(set_error(stmt->dbc,"S1106","Fetch type out of range",0));
+    DBUG_RETURN(set_stmt_error(stmt,"S1106","Fetch type out of range",0));
   }
 
   if ((long) cur_row < 0)
@@ -831,11 +877,10 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
     if (-((long) cur_row) >= (long) stmt->stmt_options.rows_in_set)
     {					/* Does not overlap start of set */
       stmt->current_row=0;		/* Fix for next fetch */
-      if (pcrow)
-        *pcrow=0;
+      *pcrow=0;
       stmt->rows_found_in_set=0;        /* Return nothing */
       mysql_data_seek(stmt->result,0L);
-      DBUG_RETURN(SQL_NO_DATA_FOUND);
+      DBUG_RETURN(SQL_SUCCESS_WITH_INFO);	/* was SQL_NO_DATA_FOUND */
     }
     /* max_row= stmt->stmt_options.rows_in_set + cur_row; */
     cur_row=0;				/* Fix seek for overlapping start */
@@ -854,8 +899,7 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
   rows_to_fetch=min(max_row-cur_row,stmt->stmt_options.rows_in_set);
   if (!rows_to_fetch)
   {
-    if (pcrow)
-      *pcrow=0;
+    *pcrow=0;
     stmt->rows_found_in_set=0;
     DBUG_RETURN(SQL_NO_DATA_FOUND);
   }
@@ -884,7 +928,10 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
       stmt->current_values=values;
     }
     if (rgfRowStatus)
+    {
       rgfRowStatus[i]=SQL_ROW_SUCCESS;
+      stmt->rgfRowStatus = rgfRowStatus;
+    }
     if (stmt->bind)
     {				/* Should always be true */
       ulong *lengths=stmt->result_lengths;
@@ -933,8 +980,7 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
     cur_row++;
   }
   stmt->rows_found_in_set=i;
-  if (pcrow)
-    *pcrow=i;
+  *pcrow=i;
 
   if (rgfRowStatus)
     for ( ; i < stmt->stmt_options.rows_in_set ; i++)
@@ -958,52 +1004,63 @@ SQLRETURN SQL_API SQLExtendedFetch(SQLHSTMT hstmt, SQLUSMALLINT fFetchType, SQLI
 }
 
 
-//  Returns the next SQL error information.
+	/*  Returns the next SQL error information. */
 
 SQLRETURN SQL_API SQLError(SQLHENV henv, SQLHDBC hdbc, SQLHSTMT hstmt,
-			 SQLCHAR FAR *szSqlState, SQLINTEGER FAR *pfNativeError,
-			 SQLCHAR FAR *szErrorMsg, SQLSMALLINT cbErrorMsgMax,
-			 SQLSMALLINT FAR *pcbErrorMsg)
+			   SQLCHAR FAR *szSqlState,
+			   SQLINTEGER FAR *pfNativeError,
+			   SQLCHAR FAR *szErrorMsg, SQLSMALLINT cbErrorMsgMax,
+			   SQLSMALLINT FAR *pcbErrorMsg)
 {
   char *errmsg;
   SQLRETURN error;
   SQLSMALLINT tmp_size;
+  SQLCHAR     tmp_state[6];
+  SQLINTEGER  tmp_error;
   DBUG_ENTER("SQLError");
   DBUG_PRINT("enter",("szErrorMsg: %lx",szErrorMsg));
 
   if (!pcbErrorMsg)
     pcbErrorMsg= &tmp_size;
+  if (!szSqlState)
+    szSqlState= tmp_state;
+  if (!pfNativeError)
+    pfNativeError= &tmp_error;
   *pcbErrorMsg=0;
   if (!hstmt && !hdbc)
     goto no_error;
-  if (hstmt && !hdbc)
-    hdbc=(SQLHDBC) ((STMT FAR*) hstmt)->dbc;
-  if (!(errmsg=mysql_error(&((DBC FAR*) hdbc)->mysql))[0])
-    goto no_error;
-  if (szSqlState)
+  if (hstmt)
+  {
+    errmsg=((STMT FAR*) hstmt)->last_error;
+    strmov((char*) szSqlState,((STMT FAR*) hstmt)->sqlstate);
+    *pfNativeError=((STMT FAR*) hstmt)->last_errno;
+  }
+  else
+  {
+    errmsg=((DBC FAR*) hdbc)->last_error;
     strmov((char*) szSqlState,((DBC FAR*) hdbc)->sqlstate);
-  if (pfNativeError)
     *pfNativeError=mysql_errno(&((DBC FAR*) hdbc)->mysql);
+  }
+  if (!(errmsg[0]))
+    goto no_error;
   DBUG_PRINT("error",("Message: %s",errmsg));
-  if ((error=copy_result((DBC FAR*) 0,szErrorMsg,cbErrorMsgMax,pcbErrorMsg,
-       "[TCX][MyODBC]")) == SQL_SUCCESS)
+  if ((error=copy_result((DBC FAR*) 0, (STMT FAR*) 0, szErrorMsg,cbErrorMsgMax,
+			 pcbErrorMsg,
+			 "[TCX][MyODBC]")) == SQL_SUCCESS)
   {
     int start_length= *pcbErrorMsg;
     cbErrorMsgMax-= start_length;
     szErrorMsg+=    start_length;
-    error=copy_result((DBC FAR*) 0,szErrorMsg,cbErrorMsgMax,pcbErrorMsg,
-		     errmsg);
+    error=copy_result((DBC FAR*) 0,(STMT FAR*) 0, szErrorMsg, cbErrorMsgMax,
+		      pcbErrorMsg, errmsg);
     (*pcbErrorMsg)+= start_length;
   }
   errmsg[0]=0;				/* Clear for next loop */
   DBUG_RETURN(error);
 
 no_error:
-  if (cbErrorMsgMax)
-    *szErrorMsg=0;
-  if (pcbErrorMsg)
-    *pcbErrorMsg=0;			/* Some extra safety */
-  if (szSqlState)
-    strmov(szSqlState,"00000");
+  *szErrorMsg=0;
+  *pcbErrorMsg=0;			/* Some extra safety */
+  strmov(szSqlState,"00000");
   DBUG_RETURN(SQL_NO_DATA_FOUND);
 }
