@@ -49,7 +49,7 @@
   ConfigDSN should return TRUE if the requested operation succeeds and
   FALSE otherwise.  The complete prototype for ConfigDSN is:
 
-  BOOL FAR PASCAL ConfigDSN(HWND    hwndParent,
+  int FAR PASCAL ConfigDSN(HWND    hwndParent,
                             WORD    fRequest,
                             LPSTR   lpszDriver,
                             LPCSTR  lpszAttributes)
@@ -200,6 +200,7 @@ void set_dlg_items(HWND hdlg ,LPSETUPDLG lpsetupdlg)
   SetDlgItemText(hdlg, IDC_PASSWORD,	lpsetupdlg->aAttr[KEY_PASSWORD].szAttr);
   SetDlgItemText(hdlg, IDC_PORT,	lpsetupdlg->aAttr[KEY_PORT].szAttr);
   SetDlgItemText(hdlg, IDC_STMT,	lpsetupdlg->aAttr[KEY_STMT].szAttr);
+  SetDlgItemText(hdlg, IDC_ERROR,	lpsetupdlg->aAttr[KEY_ERROR].szAttr);
   flag=(ulong) atol(lpsetupdlg->aAttr[KEY_FLAG].szAttr);
 #ifdef DBUG_OFF
   flag&= ~FLAG_DEBUG;
@@ -212,7 +213,7 @@ void set_dlg_items(HWND hdlg ,LPSETUPDLG lpsetupdlg)
 
 static void INTFUNC CenterDialog (HWND   hdlg);
 int  CALLBACK ConfigDlgProc (HWND hdlg,WORD wMsg,WPARAM wParam, LPARAM lParam);
-BOOL CALLBACK SetDSNAttributes(HWND hwnd, LPSETUPDLG lpsetupdlg);
+int CALLBACK SetDSNAttributes(HWND hwnd, LPSETUPDLG lpsetupdlg);
 
 /* ConfigDSN ---------------------------------------------------------------
   Description:  ODBC Setup entry point
@@ -224,12 +225,12 @@ BOOL CALLBACK SetDSNAttributes(HWND hwnd, LPSETUPDLG lpsetupdlg);
                 lpszAttributes - data source attribute string
   Output     :  TRUE success, FALSE otherwise
 --------------------------------------------------------------------------*/
-BOOL EXPFUNC ConfigDSN (HWND	 hwnd,
-			 WORD	 fRequest,
-			 LPCSTR  lpszDriver,
-			 LPCSTR  lpszAttributes)
+int EXPFUNC ConfigDSN (HWND	 hwnd,
+		       WORD	 fRequest,
+		       LPCSTR  lpszDriver,
+		       LPCSTR  lpszAttributes)
 {
-  BOOL  fSuccess;				// Success/fail flag
+  bool  fSuccess;				// Success/fail flag
   GLOBALHANDLE hglbAttr;
   LPSETUPDLG lpsetupdlg;
 
@@ -354,7 +355,7 @@ int CALLBACK ConfigDlgProc (HWND hdlg,WORD wMsg,WPARAM wParam,LPARAM lParam)
 
     Ctl3dRegister (s_hModule);
     SetWindowLong(hdlg, DWL_USER, lParam);
-#ifdef WIN32
+#ifdef _WIN32
     Ctl3dSubclassDlg(hdlg, CTL3D_ALL);
 #else
     Ctl3dSubclassDlgEx(hdlg, CTL3D_ALL);
@@ -397,7 +398,7 @@ int CALLBACK ConfigDlgProc (HWND hdlg,WORD wMsg,WPARAM wParam,LPARAM lParam)
     return TRUE;				/* Focus was not set */
   }
 
-#ifdef WIN32
+#ifdef _WIN32
   case WM_CTLCOLORBTN:
   case WM_CTLCOLORDLG:
   case WM_CTLCOLOREDIT:
@@ -405,7 +406,7 @@ int CALLBACK ConfigDlgProc (HWND hdlg,WORD wMsg,WPARAM wParam,LPARAM lParam)
   case WM_CTLCOLORMSGBOX:
   case WM_CTLCOLORSCROLLBAR:
   case WM_CTLCOLORSTATIC:
-    return (BOOL)Ctl3dCtlColorEx(wMsg, wParam, lParam);
+    return (int) Ctl3dCtlColorEx(wMsg, wParam, lParam);
 
   case WM_SETTEXT:
   case WM_NCPAINT:
@@ -435,7 +436,7 @@ int CALLBACK ConfigDlgProc (HWND hdlg,WORD wMsg,WPARAM wParam,LPARAM lParam)
 	return TRUE;
       }
       break;
-
+		
 
     // Accept results
     case IDOK:
@@ -548,8 +549,7 @@ void INTFUNC ParseAttributes(LPCSTR lpszAttributes, LPSETUPDLG lpsetupdlg)
       strmake(lpsetupdlg->aAttr[iElement].szAttr,lpszStart,
 	      min(lpsz-lpszStart, sizeof(lpsetupdlg->aAttr[0].szAttr)-1));
     }
-    if (*lpsz)
-      lpsz++;               /* Skipp ';' */
+    lpsz++;               /* Skipp ';' or end 0 */
   }
   DBUG_VOID_RETURN;
 }
@@ -560,7 +560,7 @@ void INTFUNC ParseAttributes(LPCSTR lpszAttributes, LPSETUPDLG lpsetupdlg)
   Input      :  hwnd - Parent window handle (plus globals)
   Output     :  TRUE if successful, FALSE otherwise
 --------------------------------------------------------------------------*/
-BOOL INTFUNC SetDSNAttributes(HWND hwndParent, LPSETUPDLG lpsetupdlg)
+int INTFUNC SetDSNAttributes(HWND hwndParent, LPSETUPDLG lpsetupdlg)
 {
   LPCSTR lpszDSN=lpsetupdlg->aAttr[KEY_DSN].szAttr;  // Pointer to data source name
 
@@ -648,15 +648,15 @@ BOOL INTFUNC SetDSNAttributes(HWND hwndParent, LPSETUPDLG lpsetupdlg)
 ** Connect dialog for driver connect
 *****************************************************************************/
 
-BOOL FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
-				  LPARAM lParam)
+int FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
+				 LPARAM lParam)
 {
   LPSETUPDLG lpsetupdlg;
   switch (wMsg) {
   case WM_INITDIALOG:
     Ctl3dRegister (s_hModule);
     SetWindowLong(hdlg, DWL_USER, lParam);
-#ifdef WIN32
+#ifdef _WIN32
     Ctl3dSubclassDlg(hdlg, CTL3D_ALL);
 #else
     Ctl3dSubclassDlgEx(hdlg, CTL3D_ALL);
@@ -683,7 +683,7 @@ BOOL FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
 		       EM_LIMITTEXT, (WORD) (MAXSTMT-1), 0L);
     return TRUE;
 
-#ifdef WIN32
+#ifdef _WIN32
   case WM_CTLCOLORBTN:
   case WM_CTLCOLORDLG:
   case WM_CTLCOLOREDIT:
@@ -691,7 +691,7 @@ BOOL FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
   case WM_CTLCOLORMSGBOX:
   case WM_CTLCOLORSCROLLBAR:
   case WM_CTLCOLORSTATIC:
-    return (BOOL)Ctl3dCtlColorEx(wMsg, wParam, lParam);
+    return (int)Ctl3dCtlColorEx(wMsg, wParam, lParam);
 
   case WM_SETTEXT:
   case WM_NCPAINT:
@@ -706,6 +706,8 @@ BOOL FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
 
   case WM_COMMAND:
     switch (GET_WM_COMMAND_ID(wParam, lParam)) {
+			/* even when DSN not entered, enable the ID button */
+#if 0
     case IDC_DSNAME:
       if (GET_WM_COMMAND_CMD(wParam, lParam) == EN_CHANGE)
       {
@@ -715,6 +717,7 @@ BOOL FAR PASCAL DriverConnectProc(HWND hdlg, WORD wMsg,	WPARAM wParam,
 	return TRUE;
       }
       break;
+#endif
     case IDOK:
     {
       ulong flag;
