@@ -4,6 +4,27 @@
 #include "myodbc.h"
 #include "mysqld_error.h"
 #include "errmsg.h"
+#include "m_ctype.h"
+
+
+SQLRETURN odbc_stmt(DBC FAR *dbc, const char *query)
+{
+  SQLRETURN result=SQL_SUCCESS;
+  DBUG_ENTER("odbc_stmt");
+  DBUG_PRINT("enter",("stmt: %s",query));
+
+  pthread_mutex_lock(&dbc->lock);
+  if (check_if_server_is_alive(dbc) ||
+      mysql_real_query(&dbc->mysql,query,strlen(query)))
+  {
+    set_dbc_error(dbc,"S1000",mysql_error(&dbc->mysql),
+		  mysql_errno(&dbc->mysql));
+    result=SQL_ERROR;
+  }
+  pthread_mutex_unlock(&dbc->lock);
+  DBUG_RETURN(result);
+}
+
 
 /* Use own fields instead of sql fields */
 
@@ -24,8 +45,8 @@ void fix_result_types(STMT *stmt)
   DBUG_ENTER("fix_result_types");
 
   stmt->state=ST_EXECUTED;			/* Mark set found */
-  if ((stmt->odbc_types=(SQLSMALLINT*) my_malloc(sizeof(SQLSMALLINT)*result->field_count,
-					   MYF(0))))
+  if ((stmt->odbc_types=(SQLSMALLINT*)
+       my_malloc(sizeof(SQLSMALLINT)*result->field_count, MYF(0))))
   {
     for (i=0 ; i < result->field_count ; i++)
     {
@@ -105,17 +126,6 @@ bool empty_str(char *from,int length)
   return !length;
 }
 
-void remove_escape(char *name)
-{
-  char *to;
-  for (to=name ; *name ; name++)
-  {
-    if (*name == '\\' && name[1])
-      name++;
-    *to++= *name;
-  }
-  *to=0;
-}
 
 /*
 ** Return (possibly truncated) results
@@ -123,8 +133,9 @@ void remove_escape(char *name)
 ** length of the truncted result;
 */
 
-SQLRETURN copy_result(DBC *dbc,SQLCHAR FAR *rgbValue,SQLSMALLINT cbValueMax,
-		    SQLSMALLINT FAR *pcbValue,char FAR *src)
+SQLRETURN copy_result(DBC FAR *dbc, STMT FAR *stmt, SQLCHAR FAR *rgbValue,
+		      SQLSMALLINT cbValueMax,
+		      SQLSMALLINT FAR *pcbValue,char FAR *src)
 {
   SQLCHAR FAR *dst=rgbValue;
   SQLSMALLINT save_ValueMax=cbValueMax;
@@ -150,21 +161,26 @@ SQLRETURN copy_result(DBC *dbc,SQLCHAR FAR *rgbValue,SQLSMALLINT cbValueMax,
     return SQL_SUCCESS;			/* There was room after all */
   DBUG_PRINT("warning",("Truncated string of length: %d to %d",
 			strlen(src)+ (dst-rgbValue)+1, save_ValueMax));
-  if (dbc)
-    set_error(dbc,"01004","Data truncated",4002);
-
+  if (stmt)
+    set_stmt_error(stmt,"01004","Data truncated",4002);
+  else if (dbc)
+    set_dbc_error(dbc,"01004","Data truncated",4002);
   return SQL_SUCCESS_WITH_INFO;
 }
 
 
-SQLRETURN copy_lresult(DBC FAR *dbc,SQLCHAR FAR *rgbValue,SQLINTEGER cbValueMax,
-		     SQLINTEGER FAR *pcbValue,char *src,ulong src_length,
-		     ulong max_length,ulong fill_length,ulong *offset,
-		     my_bool binary_data)
+SQLRETURN copy_lresult(DBC FAR *dbc,SQLCHAR FAR *rgbValue,
+		       SQLINTEGER cbValueMax,
+		       SQLINTEGER FAR *pcbValue,char *src,long src_length,
+		       long max_length,long fill_length,long *offset,
+		       my_bool binary_data)
 {
   char *dst=(char*) rgbValue;
   ulong length;
 
+  if(src && src_length == SQL_NTS)
+    src_length = strlen(src);
+
   if (cbValueMax && !binary_data)		/* If not length check */
     cbValueMax--;				/* Room for end null */
   else if (!cbValueMax)
@@ -182,7 +198,7 @@ SQLRETURN copy_lresult(DBC FAR *dbc,SQLCHAR FAR *rgbValue,SQLINTEGER cbValueMax,
   else if (*offset >= fill_length)
     return SQL_NO_DATA_FOUND;
   src+= *offset ; src_length-= *offset; fill_length-= *offset;
-  length=min(fill_length,(ulong) cbValueMax);
+  length=min(fill_length, cbValueMax);
   (*offset)+=length;				/* Fix for next call */
   if (pcbValue)
     *pcbValue=fill_length;
@@ -195,12 +211,12 @@ SQLRETURN copy_lresult(DBC FAR *dbc,SQLCHAR FAR *rgbValue,SQLINTEGER cbValueMax,
     if (!binary_data || length != (ulong) cbValueMax)
       dst[length]=0;
   }
-  if (!dst || (ulong) cbValueMax >= fill_length)
+  if (!dst || cbValueMax >= fill_length)
     return SQL_SUCCESS;
   DBUG_PRINT("info",("Returned %ld characters from offset: %ld",
 		     length,*offset - length));
   if (dbc)
-    set_error(dbc,"01004","Data truncated",4002);
+    set_dbc_error(dbc,"01004","Data truncated",4002);
   return SQL_SUCCESS_WITH_INFO;
 }
 
@@ -247,18 +263,28 @@ SQLRETURN copy_binary_result(DBC FAR *dbc,SQLCHAR FAR *rgbValue,SQLINTEGER cbVal
   DBUG_PRINT("info",("Returned %ld characters from offset: %ld",
 		     length,*offset - length));
   if (dbc)
-    set_error(dbc,"01004","Data truncated",4002);
+    set_dbc_error(dbc,"01004","Data truncated",4002);
   return SQL_SUCCESS_WITH_INFO;
 }
 
 
-SQLRETURN set_error(DBC FAR *dbc,char *state,char *message,uint errcode)
+SQLRETURN set_dbc_error(DBC FAR *dbc,char *state,char *message,uint errcode)
 {
-  DBUG_ENTER("set_error");
+  DBUG_ENTER("set_dbc_error");
   DBUG_PRINT("error",("message: %s",message));
   strmov(dbc->sqlstate,state);
-  strmov(mysql_error(&dbc->mysql),message);
-  dbc->mysql.net.last_errno=errcode;
+  strmov(dbc->last_error,message);
+  dbc->last_errno=errcode;
+  DBUG_RETURN(SQL_ERROR);
+}
+
+SQLRETURN set_stmt_error(STMT FAR *stmt,char *state,char *message,uint errcode)
+{
+  DBUG_ENTER("set_dbc_error");
+  DBUG_PRINT("error",("message: %s",message));
+  strmov(stmt->sqlstate,state);
+  strmov(stmt->last_error,message);
+  stmt->last_errno=errcode;
   DBUG_RETURN(SQL_ERROR);
 }
 
@@ -266,11 +292,11 @@ SQLRETURN set_error(DBC FAR *dbc,char *state,char *message,uint errcode)
 ** Translate a MySQL error to a ODBC error
 */
 
-void translate_error(DBC FAR *dbc)
+void translate_error(char *save_state,char *default_state,uint mysql_err)
 {
-  char *state=0;
+  char *state=default_state;
   DBUG_ENTER("translate_error");
-  switch (mysql_errno(&dbc->mysql)) {
+  switch (mysql_err) {
   case ER_WRONG_VALUE_COUNT:	state= "21S01"; break;
   case ER_DUP_KEY:		state= "23000"; break;
   case ER_PARSE_ERROR:		state= "42000"; break;
@@ -284,8 +310,7 @@ void translate_error(DBC FAR *dbc)
   case CR_CONNECTION_ERROR:	state= "08S01"; break;
   default: break;
   }
-  if (state)
-    strmov(dbc->sqlstate,state);
+  strmov(save_state,state);
   DBUG_VOID_RETURN;
 }
 
@@ -295,7 +320,7 @@ void translate_error(DBC FAR *dbc)
 */
 
 int unireg_to_sql_datatype(STMT FAR *stmt, MYSQL_FIELD *field, char *buff,
-			   ulong *transfer_length,ulong *precision,
+			   ulong *transfer_length, ulong *precision,
 			   ulong *display_size)
 {
   char *pos;
@@ -307,6 +332,10 @@ int unireg_to_sql_datatype(STMT FAR *stmt, MYSQL_FIELD *field, char *buff,
 
   switch(field->type) {
   case FIELD_TYPE_DECIMAL:
+    *display_size= max(field->length,field->max_length) -
+                   test(!(field->flags & UNSIGNED_FLAG)) -
+                   test(field->decimals);
+    *precision= *display_size;
     if (buff) strmov(buff,"decimal");
     return SQL_DECIMAL;
   case FIELD_TYPE_CHAR:
@@ -339,7 +368,7 @@ int unireg_to_sql_datatype(STMT FAR *stmt, MYSQL_FIELD *field, char *buff,
   case FIELD_TYPE_LONG:
     if (buff)
     {
-      pos= strmov(buff,"int");
+      pos= strmov(buff,"integer");
       if (field->flags & UNSIGNED_FLAG)
 	strmov(pos," unsigned");
     }
@@ -623,9 +652,10 @@ ulong str_to_time(const char *str,uint length)
 ****************************************************************************/
 
 int check_if_server_is_alive(DBC FAR *dbc)
-{
+{  
   time_t seconds= (time_t) time((time_t*) 0);
   int result=0;
+  DBUG_ENTER("check_if_server_is_alive");
   if ((ulong) (seconds - dbc->last_query_time) >= CHECK_IF_ALIVE)
   {
     if (mysql_ping(&dbc->mysql) &&
@@ -633,17 +663,36 @@ int check_if_server_is_alive(DBC FAR *dbc)
       result=1;
   }
   dbc->last_query_time=seconds;
-  return result;
+  DBUG_PRINT("info",("server status:%d",result));
+  DBUG_RETURN(result);
 }
 
 
+/****************************************************************************
+** Returns the previous token in the query by eating white spaces
+** If we found the start of the string, return it.
+****************************************************************************/
+
+const char *mystr_get_prev_token(const char **query, const char *start)
+{
+  const char *pos= *query;
+  do
+  {
+    if (pos == start)
+      return (*query=start);
+  }
+  while (!isspace(*(--pos))) ;
+  *query=pos;					/* Remember pos to space */
+  return (pos+1);				/* Return found token */
+}
+
 #ifndef _UNIX_
 
 /* Define functions that dosen't exist in a dll */
 
 /* _exit is called by safemalloc, mystatic & my_malloc */
 
-#ifndef WIN32
+#ifndef __WIN__
 void exit(int exit)
 {
 	abort();
